@@ -18,8 +18,10 @@
  * Boston, MA 02110-1301, USA.
  */
 #include "combinedindexmanager.h"
-#include "jstreamsconfig.h"
+#include <strigi/strigiconfig.h>
+#include "variant.h"
 
+#include "grepindexmanager.h"
 #ifdef HAVE_CLUCENE
 #include "cluceneindexmanager.h"
 #endif
@@ -38,12 +40,13 @@
 #include "indexeddocument.h"
 #include "indexreader.h"
 #include <string>
+#include <set>
 #include <map>
 using namespace std;
-using namespace jstreams;
+using namespace Strigi;
 
 map<string, IndexManager*(*)(const char*)>
-CombinedIndexManager::getFactories() {
+CombinedIndexManager::factories() {
     map<string, IndexManager*(*)(const char*)> factories;
 #ifdef HAVE_ESTRAIER
     factories["estraier"] = createEstraierIndexManager;
@@ -57,13 +60,14 @@ CombinedIndexManager::getFactories() {
 #ifdef HAVE_SQLITE
     factories["sqlite"] = createSqliteIndexManager;
 #endif
+    factories["grep"] = createGrepIndexManager;
     return factories;
 }
 
 vector<string>
-CombinedIndexManager::getBackEnds() {
+CombinedIndexManager::backEnds() {
     vector<string> v;
-    map<string, IndexManager*(*)(const char*)> f = getFactories();
+    map<string, IndexManager*(*)(const char*)> f = factories();
     map<string, IndexManager*(*)(const char*)>::const_iterator i;
     for (i=f.begin(); i!=f.end(); ++i) {
         v.push_back(i->first);
@@ -77,20 +81,24 @@ private:
 public:
     CombinedIndexReader(CombinedIndexManager* manager) :m(manager){}
     int32_t countHits(const Query& query);
-    vector<IndexedDocument> query(const Query& q);
-    map<string, time_t> getFiles(char depth);
+    vector<IndexedDocument> query(const Query& q, int offset, int max);
+    void getHits(const Strigi::Query&, const std::vector<std::string>& fields,
+        const std::vector<Strigi::Variant::Type>& types,
+        std::vector<std::vector<Strigi::Variant> >& result, int off, int max);
+    map<string, time_t> files(char depth);
     int32_t countDocuments();
     int32_t countWords();
-    int64_t getIndexSize();
-    int64_t getDocumentId(const string& uri);
-    time_t getMTime(int64_t docid);
-    vector<pair<string,uint32_t> > getHistogram(
+    int64_t indexSize();
+    int64_t documentId(const string& uri);
+    time_t mTime(int64_t docid);
+    time_t mTime(const string& uri);
+    vector<pair<string,uint32_t> > histogram(
             const string& query, const string& fieldname,
             const string& labeltype);
-    vector<string> getFieldNames();
+    vector<string> fieldNames();
     int32_t countKeywords(const string& keywordprefix,
         const vector<string>& fieldnames);
-    vector<string> getKeywords(const string& keywordmatch,
+    vector<string> keywords(const string& keywordmatch,
         const vector<string>& fieldnames,
         uint32_t max, uint32_t offset);
 };
@@ -117,11 +125,11 @@ CombinedIndexManager::Private::~Private() {
 CombinedIndexManager::CombinedIndexManager(const string& type,
         const string& indexdir) :p(new CombinedIndexManager::Private(this)) {
     // determine the right index manager
-    map<string, IndexManager*(*)(const char*)> factories = getFactories();
+    map<string, IndexManager*(*)(const char*)> l_factories = factories();
     map<string, IndexManager*(*)(const char*)>::const_iterator f
-        = factories.find(type);
-    if (f == factories.end()) {
-        f = factories.begin();
+        = l_factories.find(type);
+    if (f == l_factories.end()) {
+        f = l_factories.begin();
     }
     p->writermanager = f->second(indexdir.c_str());
 }
@@ -129,22 +137,22 @@ CombinedIndexManager::~CombinedIndexManager() {
     delete p;
 }
 IndexReader*
-CombinedIndexManager::getIndexReader() {
+CombinedIndexManager::indexReader() {
     return &(p->reader);
 }
 IndexWriter*
-CombinedIndexManager::getIndexWriter() {
-    return p->writermanager->getIndexWriter();
+CombinedIndexManager::indexWriter() {
+    return p->writermanager->indexWriter();
 }
 void
 CombinedIndexManager::addReadonlyIndex(const string& indexdir,
         const string& type) {
     removeReadonlyIndex(indexdir);
     // determine the right index manager
-    map<string, IndexManager*(*)(const char*)> factories = getFactories();
+    map<string, IndexManager*(*)(const char*)> l_factories = factories();
     map<string, IndexManager*(*)(const char*)>::const_iterator f
-        = factories.find(type);
-    if (f == factories.end()) {
+        = l_factories.find(type);
+    if (f == l_factories.end()) {
         return;
     }
     IndexManager* im = f->second(indexdir.c_str());
@@ -160,106 +168,149 @@ CombinedIndexManager::removeReadonlyIndex(const string& indexdir) {
 }
 int32_t
 CombinedIndexReader::countHits(const Query& query) {
-    int32_t c = m->p->writermanager->getIndexReader()->countHits(query);
+    int32_t c = m->p->writermanager->indexReader()->countHits(query);
     STRIGI_MUTEX_LOCK(&m->p->lock.lock);
     map<string, TSSPtr<IndexManager> > f = m->p->readmanagers;
     STRIGI_MUTEX_UNLOCK(&m->p->lock.lock);
     map<string, TSSPtr<IndexManager> >::const_iterator i;
     for (i = f.begin(); i != f.end(); ++i) {
-        c += i->second->getIndexReader()->countHits(query);
+        c += i->second->indexReader()->countHits(query);
     }
     return c;
 }
 vector<IndexedDocument>
-CombinedIndexReader::query(const Query& q) {
+CombinedIndexReader::query(const Query& q, int offset, int max) {
     /** TODO merge the result documents by score **/
-    vector<IndexedDocument> v = m->p->writermanager->getIndexReader()
-        ->query(q);
+    vector<IndexedDocument> v = m->p->writermanager->indexReader()
+        ->query(q, offset, max);
     STRIGI_MUTEX_LOCK(&m->p->lock.lock);
     map<string, TSSPtr<IndexManager> > f = m->p->readmanagers;
     STRIGI_MUTEX_UNLOCK(&m->p->lock.lock);
     map<string, TSSPtr<IndexManager> >::const_iterator i;
     for (i = f.begin(); i != f.end(); ++i) {
-        v = i->second->getIndexReader()->query(q);
+        v = i->second->indexReader()->query(q, offset, max);
     }
     return v;
 }
-map<string, time_t>
-CombinedIndexReader::getFiles(char depth) {
-    map<string, time_t> files = m->p->writermanager->getIndexReader()
-        ->getFiles(depth);
+void
+CombinedIndexReader::getHits(const Query& q,
+        const std::vector<std::string>& fields,
+        const std::vector<Strigi::Variant::Type>& types,
+        std::vector<std::vector<Variant> >& result, int off, int max) {
+#ifdef ENABLE_NEWXESAM	
+    /** TODO merge the result documents by score **/
+    std::vector<std::vector<Variant> > v;
+    m->p->writermanager->indexReader()->getHits(q, fields, types,
+        result, off, max);
     STRIGI_MUTEX_LOCK(&m->p->lock.lock);
     map<string, TSSPtr<IndexManager> > f = m->p->readmanagers;
     STRIGI_MUTEX_UNLOCK(&m->p->lock.lock);
     map<string, TSSPtr<IndexManager> >::const_iterator i;
     for (i = f.begin(); i != f.end(); ++i) {
-        files = i->second->getIndexReader()->getFiles(depth);
+        i->second->indexReader()->getHits(q, fields, types, v, off, max);
+    }
+#endif    
+}
+map<string, time_t>
+CombinedIndexReader::files(char depth) {
+    map<string, time_t> files = m->p->writermanager->indexReader()
+        ->files(depth);
+    STRIGI_MUTEX_LOCK(&m->p->lock.lock);
+    map<string, TSSPtr<IndexManager> > f = m->p->readmanagers;
+    STRIGI_MUTEX_UNLOCK(&m->p->lock.lock);
+    map<string, TSSPtr<IndexManager> >::const_iterator i;
+    for (i = f.begin(); i != f.end(); ++i) {
+        map<string, time_t> af(i->second->indexReader()->files(depth));
+        map<string, time_t>::const_iterator j;
+        for (j = af.begin(); j != af.end(); ++j) {
+            // insert entry if it does not yet occur in the map
+            files.insert(*j);
+        }
     }
     return files;
 }
 int32_t
 CombinedIndexReader::countDocuments() {
-    int32_t c = m->p->writermanager->getIndexReader()->countDocuments();
+    int32_t c = m->p->writermanager->indexReader()->countDocuments();
     STRIGI_MUTEX_LOCK(&m->p->lock.lock);
     map<string, TSSPtr<IndexManager> > f = m->p->readmanagers;
     STRIGI_MUTEX_UNLOCK(&m->p->lock.lock);
     map<string, TSSPtr<IndexManager> >::const_iterator i;
     for (i = f.begin(); i != f.end(); ++i) {
-        c += i->second->getIndexReader()->countDocuments();
+        c += i->second->indexReader()->countDocuments();
     }
     return c;
 }
 int32_t
 CombinedIndexReader::countWords() {
-    int32_t c = m->p->writermanager->getIndexReader()->countWords();
+    int32_t c = m->p->writermanager->indexReader()->countWords();
     STRIGI_MUTEX_LOCK(&m->p->lock.lock);
     map<string, TSSPtr<IndexManager> > f = m->p->readmanagers;
     STRIGI_MUTEX_UNLOCK(&m->p->lock.lock);
     map<string, TSSPtr<IndexManager> >::const_iterator i;
     for (i = f.begin(); i != f.end(); ++i) {
-        c += i->second->getIndexReader()->countWords();
+        c += i->second->indexReader()->countWords();
     }
     return c;
 }
 int64_t
-CombinedIndexReader::getIndexSize() {
-    int64_t c = m->p->writermanager->getIndexReader()->getIndexSize();
+CombinedIndexReader::indexSize() {
+    int64_t c = m->p->writermanager->indexReader()->indexSize();
     STRIGI_MUTEX_LOCK(&m->p->lock.lock);
     map<string, TSSPtr<IndexManager> > f = m->p->readmanagers;
     STRIGI_MUTEX_UNLOCK(&m->p->lock.lock);
     map<string, TSSPtr<IndexManager> >::const_iterator i;
     for (i = f.begin(); i != f.end(); ++i) {
-        c += i->second->getIndexReader()->getIndexSize();
+        c += i->second->indexReader()->indexSize();
     }
     return c;
 }
 vector<pair<string,uint32_t> >
-CombinedIndexReader::getHistogram(const string& query, const string& fieldname,
+CombinedIndexReader::histogram(const string& query, const string& fieldname,
         const string& labeltype){
-    return m->p->writermanager->getIndexReader()->getHistogram(query,
+    return m->p->writermanager->indexReader()->histogram(query,
         fieldname, labeltype);
 }
 int64_t
-CombinedIndexReader::getDocumentId(const string& uri) {
-    return m->p->writermanager->getIndexReader()->getDocumentId(uri);
+CombinedIndexReader::documentId(const string& uri) {
+    return m->p->writermanager->indexReader()->documentId(uri);
 }
 time_t
-CombinedIndexReader::getMTime(int64_t docid) {
-    return m->p->writermanager->getIndexReader()->getMTime(docid);
+CombinedIndexReader::mTime(int64_t docid) {
+    return m->p->writermanager->indexReader()->mTime(docid);
+}
+time_t
+CombinedIndexReader::mTime(const std::string& uri) {
+    return m->p->writermanager->indexReader()->mTime(uri);
 }
 vector<string>
-CombinedIndexReader::getFieldNames() {
-    return m->p->writermanager->getIndexReader()->getFieldNames();
+CombinedIndexReader::fieldNames() {
+    vector<string> n = m->p->writermanager->indexReader()->fieldNames();
+    STRIGI_MUTEX_LOCK(&m->p->lock.lock);
+    map<string, TSSPtr<IndexManager> > f = m->p->readmanagers;
+    STRIGI_MUTEX_UNLOCK(&m->p->lock.lock);
+    if (f.size() == 0) return n;
+
+    set<string> names;
+    copy(n.begin(), n.end(), inserter(names, names.begin()));
+    map<string, TSSPtr<IndexManager> >::const_iterator i;
+    for (i = f.begin(); i != f.end(); ++i) {
+        n =  i->second->indexReader()->fieldNames();
+        copy(n.begin(), n.end(), inserter(names, names.begin()));
+    }
+    n.clear();
+    copy(names.begin(), names.end(), n.begin());
+    return n;
 }
 int32_t
 CombinedIndexReader::countKeywords(const string& keywordprefix,
         const vector<string>& fieldnames) {
-    return m->p->writermanager->getIndexReader()->countKeywords(keywordprefix,
+    return m->p->writermanager->indexReader()->countKeywords(keywordprefix,
         fieldnames);
 }
 vector<string>
-CombinedIndexReader::getKeywords(const string& keywordmatch,
+CombinedIndexReader::keywords(const string& keywordmatch,
         const vector<string>& fieldnames, uint32_t max, uint32_t offset) {
-    return m->p->writermanager->getIndexReader()->getKeywords(keywordmatch,
+    return m->p->writermanager->indexReader()->keywords(keywordmatch,
         fieldnames, max, offset);
 }
