@@ -21,47 +21,28 @@
 #include "indexmanager.h"
 #include "indexreader.h"
 #include "indexwriter.h"
+#include "diranalyzer.h"
 
-#include "jstreamsconfig.h"
+#include <strigi/strigiconfig.h>
 #include "event.h"
 #include "eventlistenerqueue.h"
-#include "indexerconfiguration.h"
+#include "analyzerconfiguration.h"
 
 #include "filelister.h"
-#include "streamindexer.h"
+#include "streamanalyzer.h"
 #include <cerrno>
 #include <sys/resource.h>
 
 #include "strigilogging.h"
 
 using namespace std;
-using namespace jstreams;
-
-IndexScheduler* sched;
+using namespace Strigi;
 
 IndexScheduler::IndexScheduler() :StrigiThread("IndexScheduler") {
-    sched = this;
     m_listenerEventQueue = NULL;
 }
 IndexScheduler::~IndexScheduler() {
 }
-void
-IndexScheduler::addFileCallback(const char* path, uint dirlen, uint len,
-        time_t mtime) {
-    // only read files that do not start with '.'
-    if (path[dirlen] == '.') return;
-
-    std::string filepath(path, len);
-
-    map<string, time_t>::iterator i = sched->dbfiles.find(filepath);
-    // if the file has not yet been indexed or if the mtime has changed,
-    // put it in the list to index
-    if (i == sched->dbfiles.end() || i->second != mtime) {
-        sched->toindex[filepath] = mtime;
-    } else {
-        sched->dbfiles.erase(i);
-    }
-}
 std::string
 IndexScheduler::getStateString() {
     State state = getState();
@@ -71,7 +52,7 @@ IndexScheduler::getStateString() {
 }
 int
 IndexScheduler::getQueueSize() {
-    return toindex.size();
+    return 0;
 }
 void
 shortsleep(long nanoseconds) {
@@ -109,76 +90,38 @@ IndexScheduler::run(void*) {
 }
 void
 IndexScheduler::index() {
-    IndexReader* reader = indexmanager->getIndexReader();
-    IndexWriter* writer = indexmanager->getIndexWriter();
-    StreamIndexer* streamindexer = new StreamIndexer(*writer,
-        *m_indexerconfiguration);
+    Strigi::IndexWriter* writer = indexmanager->indexWriter();
+    DirAnalyzer analyzer(*indexmanager, *m_indexerconfiguration);
+    vector<std::string> dirs;
+    copy(dirstoindex.begin(), dirstoindex.end(), back_inserter(dirs));
+    analyzer.updateDirs(dirs, 2, this);
 
-    if (dbfiles.size() == 0 && toindex.size() == 0) {
-        // retrieve the list of real files currently in the database
-        dbfiles = reader->getFiles(0);
-
-        char buff [20];
-        snprintf(buff, 20* sizeof (char), "%i", dbfiles.size());
-        STRIGI_LOG_DEBUG ("strigi.IndexScheduler", string(buff) + " real files in the database")
-
-        // first loop through all files
-        FileLister lister (*m_indexerconfiguration);
-        lister.setFileCallbackFunction(&addFileCallback);
-        STRIGI_LOG_DEBUG ("strigi.IndexScheduler", "going to index")
-        set<string>::const_iterator i;
-        for (i = dirstoindex.begin(); i != dirstoindex.end(); ++i) {
-            lister.listFiles(i->c_str());
-        }
-
-        snprintf(buff, 20* sizeof (char), "%i", dbfiles.size());
-        STRIGI_LOG_DEBUG ("strigi.IndexScheduler", string(buff) + " files to remove")
-
-        snprintf(buff, 20* sizeof (char), "%i", toindex.size());
-        STRIGI_LOG_DEBUG ("strigi.IndexScheduler", string(buff) + " files to add or update")
-    }
-
-    vector<string> todelete;
-    map<string,time_t>::iterator it = dbfiles.begin();
-    while (getState() == Working && it != dbfiles.end()) {
-        todelete.push_back(it->first);
-//        writer->deleteEntry(it->first);
-        dbfiles.erase(it++);
-    }
-    writer->deleteEntries(todelete);
-
-    it = toindex.begin();
-    while (getState() == Working && it != toindex.end()) {
-        streamindexer->indexFile(it->first);
-        if (writer->itemsInCache() > 10000) {
-            writer->commit();
-        }
-        toindex.erase(it++);
-    }
+    writer->commit();
     if (getState() == Working) {
-        writer->commit();
         writer->optimize();
     }
-
-    delete streamindexer;
 }
 
 void
 IndexScheduler::processListenerEvents(vector<Event*>& events) {
-    IndexReader* reader = indexmanager->getIndexReader();
-    IndexWriter* writer = indexmanager->getIndexWriter();
-    IndexerConfiguration ic;
-    StreamIndexer* streamindexer = new StreamIndexer(*writer, ic);
+    Strigi::IndexReader* reader = indexmanager->indexReader();
+    Strigi::IndexWriter* writer = indexmanager->indexWriter();
+    Strigi::AnalyzerConfiguration ic;
+    Strigi::StreamAnalyzer* streamindexer = new Strigi::StreamAnalyzer(ic);
+    streamindexer->setIndexWriter(*writer);
 
+    map<string, time_t> toindex;
     vector<string> toDelete;
 
-    STRIGI_LOG_DEBUG ("strigi.IndexScheduler", "processing listener's events")
+    STRIGI_LOG_DEBUG ("strigi.IndexScheduler", "processing listener's events");
 
-    for(vector<Event*>::iterator iter = events.begin(); iter != events.end(); iter++)
+    for(vector<Event*>::iterator iter = events.begin();
+        iter != events.end(); iter++)
     {
         Event* event = *iter;
 
-        STRIGI_LOG_DEBUG ("strigi.IndexScheduler", "event infos: " + event->toString())
+        STRIGI_LOG_DEBUG ("strigi.IndexScheduler",
+                          "event infos: " + event->toString());
 
         switch (event->getType())
         {
@@ -187,14 +130,17 @@ IndexScheduler::processListenerEvents(vector<Event*>& events) {
                 break;
             case Event::UPDATED:
             {
-                time_t indexTime = reader->getMTime(reader->getDocumentId(event->getPath()));
+                time_t indexTime = reader->mTime(event->getPath());
+
                 if (indexTime < event->getTime())
                 {
-                    toindex.insert (make_pair (event->getPath(), event->getTime()));
+                    toindex.insert (make_pair (event->getPath(),
+                                               event->getTime()));
                     toDelete.push_back (event->getPath());
                 }
                 else
-                    STRIGI_LOG_DEBUG ("strigi.IndexScheduler", "ignoring last event")
+                    STRIGI_LOG_DEBUG ("strigi.IndexScheduler",
+                                      "ignoring last event");
                 break;
             }
             case Event::DELETED:
@@ -249,6 +195,6 @@ IndexScheduler::setIndexedDirectories(const std::set<std::string> &d) {
         }
     }
     if (dirstoindex.size() == 0) {
-        indexmanager->getIndexWriter()->deleteAllEntries();
+        indexmanager->indexWriter()->deleteAllEntries();
     }
 }
