@@ -25,46 +25,41 @@
 #include <set>
 #include <vector>
 #include "strigithread.h"
+#include "diranalyzer.h"
 
 class Event;
 class EventListenerQueue;
 
-namespace jstreams {
+namespace Strigi {
     class IndexManager;
-    class IndexerConfiguration;
+    class AnalyzerConfiguration;
 }
-class IndexScheduler : public StrigiThread {
-friend bool addFileCallback(const char* path, uint dirlen, uint len,
-        time_t mtime);
+class IndexScheduler : public StrigiThread, private Strigi::AnalysisCaller {
 private:
     std::set<std::string> dirstoindex;
-    jstreams::IndexManager* indexmanager;
-    std::map<std::string, time_t> dbfiles;
-    std::map<std::string, time_t> toindex;
+    Strigi::IndexManager* indexmanager;
 
     EventListenerQueue* m_listenerEventQueue;
-    jstreams::IndexerConfiguration* m_indexerconfiguration;
+    Strigi::AnalyzerConfiguration* m_indexerconfiguration;
     void processListenerEvents(std::vector<Event*>& events);
 
     void* run(void*);
     void index();
 public:
-    static void addFileCallback(const char* path, uint dirlen, uint len,
-        time_t mtime);
     IndexScheduler();
     void addDirToIndex(const std::string& d) {
         dirstoindex.insert(d);
     }
-    void setIndexManager(jstreams::IndexManager* m) {
+    void setIndexManager(Strigi::IndexManager* m) {
         indexmanager = m;
     }
     void setEventListenerQueue (EventListenerQueue* eventQueue) {
         m_listenerEventQueue = eventQueue;
     }
-    void setIndexerConfiguration(jstreams::IndexerConfiguration* ic) {
+    void setIndexerConfiguration(Strigi::AnalyzerConfiguration* ic) {
         m_indexerconfiguration = ic;
     }
-    jstreams::IndexerConfiguration& getIndexerConfiguration() const {
+    Strigi::AnalyzerConfiguration& getIndexerConfiguration() const {
         return *m_indexerconfiguration;
     }
     int getQueueSize();
@@ -76,6 +71,9 @@ public:
         return dirstoindex;
     }
     void setIndexedDirectories(const std::set<std::string> &d);
+    bool continueAnalysis() {
+        return getState() == Working;
+    }
 };
 
 #endif
