@@ -17,40 +17,69 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
-#include "jstreamsconfig.h"
-#include "grepindexwriter.h"
-#include "jstreamsconfig.h"
-#include "indexer.h"
-#include "indexerconfiguration.h"
+#include <strigi/strigiconfig.h>
+#include "grepindexmanager.h"
+#include "diranalyzer.h"
+#include "analyzerconfiguration.h"
+#include <iostream>
+#include <cstring>
+using namespace Strigi;
+using namespace std;
 
 void
 printUsage(char** argv) {
-    fprintf(stderr, "Usage: %s PATTERN [dir-or-file-to-grep]\n", argv[0]);
+    fprintf(stderr, "Usage: %s [--fields] [--help] PATTERN [dir-or-file-to-grep]\n"
+        " --fields print the list of fields\n"
+        " --help   print this help screen\n",
+        argv[0]);
+}
+bool
+containsArgument(int argc, char **argv, const char* arg, const char* a=0) {
+    for (int i=1; i<argc; ++i) {
+         if (strcmp(argv[i], arg) == 0
+             || (a && strcmp(argv[i], a) == 0)) return true;
+    }
+    return false;
 }
 bool
 containsHelp(int argc, char **argv) {
-    for (int i=1; i<argc; ++i) {
-         if (strcmp(argv[i], "--help") == 0
-             || strcmp(argv[i], "-h") == 0) return true;
+    return containsArgument(argc, argv, "--help", "-h");
+}
+bool
+containsFieldList(int argc, char **argv) {
+    return containsArgument(argc, argv, "--fields", "-f");
+}
+
+void
+printFields(AnalyzerConfiguration& conf) {
+    const map<string, RegisteredField*>& fields
+        = conf.fieldRegister().fields();
+    map<string, RegisteredField*>::const_iterator i;
+    for (i = fields.begin(); i != fields.end(); ++i) {
+        cout << i->first << endl;
     }
-    return false;
 }
 
 int
 main(int argc, char** argv) {
+    AnalyzerConfiguration ic;
+    if (containsFieldList(argc, argv)) {
+        printFields(ic);
+        return 0;
+    }
     if (containsHelp(argc, argv) || argc < 2) {
         printUsage(argv);
         return -1;
     }
-    GrepIndexWriter writer(argv[1]);
-    jstreams::IndexerConfiguration ic;
-    Indexer indexer(writer, ic);
+    GrepIndexManager manager(argv[1]);
+
+    DirAnalyzer analyzer(manager, ic);
     if (argc > 2) {
         for (int32_t i=2; i<argc; ++i) {
-            indexer.index(argv[i]);
+            analyzer.analyzeDir(argv[i]);
         }
     } else {
-        indexer.index(".");
+        analyzer.analyzeDir(".");
     }
     return 0;
 }
