@@ -17,10 +17,12 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
-#include "jstreamsconfig.h"
-#include "indexer.h"
+#include <strigi/strigiconfig.h>
+#include "diranalyzer.h"
 #include "dummyindexwriter.h"
-#include "indexerconfiguration.h"
+#include "analyzerconfiguration.h"
+
+#include <stdlib.h>
 
 void
 printUsage(char** argv) {
@@ -42,9 +44,9 @@ main(int argc, char **argv) {
         verbosity = atoi(argv[2]);
     }
 
-    DummyIndexWriter writer(verbosity);
-    jstreams::IndexerConfiguration ic;
-    Indexer indexer(writer, ic);
-    indexer.index(argv[argc-1]);
+    DummyIndexManager manager(verbosity);
+    Strigi::AnalyzerConfiguration ic;
+    Strigi::DirAnalyzer analyzer(manager, ic);
+    analyzer.analyzeDir(argv[argc-1]);
     return 0;
 }
