@@ -19,11 +19,12 @@
  */
 #include "estraierindexwriter.h"
 #include "estraierindexmanager.h"
-#include "indexable.h"
+#include "analysisresult.h"
+#include <fieldtypes.h>
 #include <vector>
 #include <sstream>
 using namespace std;
-using namespace jstreams;
+using namespace Strigi;
 
 struct EstraierData {
     ESTDOC* doc;
@@ -38,26 +39,26 @@ EstraierIndexWriter::~EstraierIndexWriter() {
     commit();
 }
 void
-EstraierIndexWriter::addText(const Indexable* idx, const char* text,
+EstraierIndexWriter::addText(const AnalysisResult* idx, const char* text,
         int32_t length) {
-    ESTDOC* doc = static_cast<ESTDOC*>(idx->getWriterData());;
+    ESTDOC* doc = static_cast<ESTDOC*>(idx->writerData());;
     string value(text, length);
     est_doc_add_text(doc, value.c_str());
 }
 void
-EstraierIndexWriter::setField(const Indexable* idx, const string& name,
-        const string& value) {
-    ESTDOC* doc = static_cast<ESTDOC*>(idx->getWriterData());
-    if (name == "size") {
+EstraierIndexWriter::addValue(const AnalysisResult* idx,
+        const RegisteredField* field, const string& value) {
+    ESTDOC* doc = static_cast<ESTDOC*>(idx->writerData());
+    if (field->getKey() == "size") {
         est_doc_add_attr(doc, "@size", value.c_str());
-    } else if (name == "title") {
+    } else if (field->getKey() == "title") {
         est_doc_add_attr(doc, "@title", value.c_str());
     } else {
-        est_doc_add_attr(doc, name.c_str(), value.c_str());
+        est_doc_add_attr(doc, field->getKey().c_str(), value.c_str());
     }
 }
 void
-EstraierIndexWriter::startIndexable(Indexable* idx) {
+EstraierIndexWriter::startAnalysis(AnalysisResult* idx) {
     // allocate a new estraier document
     ESTDOC* doc = est_doc_new();
     idx->setWriterData(doc);
@@ -66,15 +67,15 @@ EstraierIndexWriter::startIndexable(Indexable* idx) {
     Close all left open indexwriters for this path.
 */
 void
-EstraierIndexWriter::finishIndexable(const Indexable* idx) {
-    ESTDOC* doc = static_cast<ESTDOC*>(idx->getWriterData());;
+EstraierIndexWriter::finishAnalysis(const AnalysisResult* idx) {
+    ESTDOC* doc = static_cast<ESTDOC*>(idx->writerData());;
     // add required url field
 
-    est_doc_add_attr(doc, "@uri", idx->getPath().c_str());
+    est_doc_add_attr(doc, "@uri", idx->path().c_str());
     char numbuf[64];
-    sprintf(numbuf, "%llu", (int64_t)idx->getMTime());
+    sprintf(numbuf, "%llu", (int64_t)idx->mTime());
     est_doc_add_attr(doc, "@mdate", numbuf);
-    sprintf(numbuf, "%i", idx->getDepth());
+    sprintf(numbuf, "%i", idx->depth());
     est_doc_add_attr(doc, "depth", numbuf);
 
     ESTDB* db = manager->ref();
