@@ -1,9 +1,9 @@
-#include "jstreamsconfig.h"
+#include <strigi/strigiconfig.h>
 #include "estraierindexmanager.h"
 #include "indexmanagertests.h"
 #include "indexwritertests.h"
 #include "indexreadertests.h"
-#include "indexerconfiguration.h"
+#include "analyzerconfiguration.h"
 #include <sys/stat.h>
 #include <sys/types.h>
 
@@ -17,16 +17,16 @@ EstraierTest(int argc, char**argv) {
     mkdir(path, S_IRUSR|S_IWUSR|S_IXUSR);
     EstraierIndexManager* manager = new EstraierIndexManager(path);
 
-    jstreams::IndexerConfiguration ic;
+    Strigi::AnalyzerConfiguration ic;
     IndexManagerTests tests(manager, ic);
     tests.testAll();
     tests.testAllInThreads(20);
 
-    jstreams::IndexWriter* writer = manager->getIndexWriter();
+    Strigi::IndexWriter* writer = manager->indexWriter();
     IndexWriterTests wtests(*writer, ic);
     wtests.testAll();
 
-    jstreams::IndexReader* reader = manager->getIndexReader();
+    Strigi::IndexReader* reader = manager->indexReader();
     IndexReaderTests rtests(reader);
     rtests.testAll();
 
