@@ -21,21 +21,25 @@
 #include "socketclient.h"
 #include "indexreader.h"
 #include "query.h"
+#include "queryparser.h"
 #include <unistd.h>
 #include <sys/types.h>
 #include <dirent.h>
 #include <sstream>
 #include <fstream>
 #include <sys/stat.h>
+#include <stdlib.h>
+#include <string.h>
+
 using namespace std;
-using namespace jstreams;
+using namespace Strigi;
 
 class StrigiHtmlGui::Private {
 private:
     HtmlHelper* h;
     string highlightTerms(const string& t, const Query& q) const;
     void printSearchResult(ostream& out,
-        const jstreams::IndexedDocument& doc, const Query& q) const;
+        const Strigi::IndexedDocument& doc, const Query& q) const;
 public:
     SocketClient strigi;
 
@@ -210,10 +214,10 @@ StrigiHtmlGui::printSearch(ostream& out, const string& path,
         bool doother = true;
         tabs = readTabQueries();
         if (tabs.size() == 0) {
-            tabs["Images"] = "mimetype:image*";
-            tabs["Mail"] = "mimetype:message/*";
-            tabs["Web"] = "mimetype:text/html";
-            tabs["Text"] = "mimetype:text/*";
+            tabs["Images"] = "content.mime_type:image*";
+            tabs["Mail"] = "content.mime_type:message/*";
+            tabs["Web"] = "content.mime_type:text/html";
+            tabs["Text"] = "content.mime_type:text/*";
         }
         map<string, string>::const_iterator j;
         string otherq = query;
@@ -293,26 +297,30 @@ StrigiHtmlGui::printSearch(ostream& out, const string& path,
         out << "</div>";
     }
 }
-set<string>
-getTerms(const Query& q) {
-    set<string> terms;
-    if (q.getOccurrence() == Query::MUST_NOT) {
-        return terms;
+void
+getFields(set<string>& fields, const Query& query) {
+    copy(query.fields().begin(), query.fields().end(),
+        inserter(fields, fields.begin()));
+    for (vector<Query>::const_iterator i = query.subQueries().begin();
+            i != query.subQueries().end(); ++i) {
+        getFields(fields, *i);
     }
-    if (q.getExpression().size() > 0) {
-        terms.insert(q.getExpression());
+}
+void
+getTerms(set<string>& terms, const Query& q) {
+    if (q.term().string().size() && !q.negate()) {
+        terms.insert(q.term().string());
     }
-    list<Query>::const_iterator j;
-    for (j = q.getTerms().begin(); j != q.getTerms().end(); ++j) {
-        set<string> t = getTerms(*j);
-        copy(t.begin(), t.end(), inserter(terms, terms.begin()));
+    for (vector<Query>::const_iterator i = q.subQueries().begin();
+            i != q.subQueries().end(); ++i) {
+        getTerms(terms, *i);
     }
-    return terms;
 }
 string
 StrigiHtmlGui::Private::highlightTerms(const string& t, const Query& q) const {
     vector<string> terms;
-    set<string> termset = getTerms(q);
+    set<string> termset;
+    getTerms(termset, q);
     copy(termset.begin(), termset.end(), back_inserter(terms));
     string out = h->highlight(t, terms);
     return out;
@@ -401,8 +409,8 @@ toSizeString(int s) {
 }
 void
 StrigiHtmlGui::Private::printSearchResult(ostream& out,
-        const jstreams::IndexedDocument& doc, const Query& query) const {
-    map<string, string>::const_iterator t;
+        const Strigi::IndexedDocument& doc, const Query& query) const {
+    multimap<string, string>::const_iterator t;
     string link, icon, name, folder;
     int depth = 0;
     t = doc.properties.find("depth");
@@ -415,7 +423,10 @@ StrigiHtmlGui::Private::printSearchResult(ostream& out,
         icon = "<div class='iconbox'><img class='icon' src='"+icon;
         icon += "'/></div>\n";
     }
-    t = doc.properties.find("title");
+    t = doc.properties.find("audio.title");
+    if (t == doc.properties.end()) {
+        t = doc.properties.find("email.subject");
+    }
     size_t l = doc.uri.rfind('/');
     if (t != doc.properties.end()) {
         name = t->second.c_str();
@@ -472,8 +483,8 @@ void
 StrigiHtmlGui::Private::printSearchResults(ostream& out,
         const ClientInterface::Hits& hits, const string& q) const {
     QueryParser parser;
-    Query query = parser.buildQuery(q, 0, 0);
-    vector<jstreams::IndexedDocument>::const_iterator i;
+    Query query = parser.buildQuery(q);
+    vector<Strigi::IndexedDocument>::const_iterator i;
     for (i = hits.hits.begin(); i != hits.hits.end(); ++i) {
         printSearchResult(out, *i, query);
     }
