@@ -17,16 +17,23 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
-#include "jstreamsconfig.h"
-#include "query.h"
-#include "textutils.h"
-#include <CLucene.h>
+
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 #include "cluceneindexreader.h"
+#include <strigi/strigiconfig.h>
+#include "query.h"
+#include "queryparser.h"
+#include "variant.h"
+#include "textutils.h"
 #include "cluceneindexmanager.h"
 #include "timeofday.h"
 #include <CLucene/search/QueryFilter.h>
 #include <CLucene/index/Terms.h>
 #include <sstream>
+#include <iostream>
 
 using lucene::search::Hits;
 using lucene::search::IndexSearcher;
@@ -38,47 +45,65 @@ using lucene::index::TermEnum;
 using lucene::search::TermQuery;
 using lucene::search::WildcardQuery;
 using lucene::search::BooleanQuery;
+using lucene::search::Query;
 using lucene::search::RangeQuery;
 using lucene::search::QueryFilter;
 using lucene::search::HitCollector;
 using lucene::util::BitSet;
 using lucene::document::DocumentFieldEnumeration;
-using namespace jstreams;
+using Strigi::IndexedDocument;
+using Strigi::Variant;
 
 class HitCounter : public HitCollector {
 private:
-    int32_t count;
-    void collect (const int32_t doc, const float_t score) { count++; }
+    int32_t m_count;
+    void collect (const int32_t doc, const float_t score) { m_count++; }
 public:
-    HitCounter() :count(0) {}
-    int32_t getCount() const { return count; }
+    HitCounter() :m_count(0) {}
+    int32_t count() const { return m_count; }
 };
 
 class CLuceneIndexReader::Private {
 public:
+    CLuceneIndexReader& reader;
+    Private(CLuceneIndexReader& r) :reader(r) {}
+
     static Term* createTerm(const wchar_t* name, const string& value);
+    static Term* createKeywordTerm(const wchar_t* name, const string& value);
     static Term* createWildCardTerm(const wchar_t* name, const string& value);
-    static BooleanQuery* createBooleanQuery(const Query& query);
+    Query* createQuery(const Strigi::Query& query);
+    Query* createSimpleQuery(const Strigi::Query& query);
+    static Query* createSingleFieldQuery(const string& field,
+        const Strigi::Query& query);
+    Query* createNoFieldQuery(const Strigi::Query& query);
+    Query* createMultiFieldQuery(const Strigi::Query& query);
+    BooleanQuery* createBooleanQuery(const Strigi::Query& query);
     static void addField(lucene::document::Field* field, IndexedDocument&);
+    Variant getFieldValue(lucene::document::Field* field, Variant::Type) const;
 };
 
 CLuceneIndexReader::CLuceneIndexReader(CLuceneIndexManager* m,
-    const string& dir) :manager(m), dbdir(dir), otime(0), reader(0) {
+    const string& dir) :manager(m), p(new Private(*this)), dbdir(dir), otime(0),
+        reader(0) {
     openReader();
 }
 
 CLuceneIndexReader::~CLuceneIndexReader() {
     closeReader();
+    delete p;
 }
 void
 CLuceneIndexReader::openReader() {
+    closeReader();
     doccount = -1;
     wordcount = -1;
     try {
-//        printf("reader at %s\n", dbdir.c_str());
         reader = lucene::index::IndexReader::open(dbdir.c_str());
+        //fprintf(stderr,
+        //"reader at %s: %i\n", dbdir.c_str(), reader->numDocs());
     } catch (CLuceneError& err) {
-        printf("could not create reader: %s\n", err.what());
+        fprintf(stderr, "could not create reader %s: %s\n", dbdir.c_str(),
+            err.what());
         reader = 0;
     }
 }
@@ -88,14 +113,14 @@ CLuceneIndexReader::closeReader() {
     try {
         reader->close();
     } catch (CLuceneError& err) {
-        printf("could not close clucene: %s\n", err.what());
+        fprintf(stderr, "could not close clucene: %s\n", err.what());
     }
     delete reader;
     reader = 0;
 }
 bool
 CLuceneIndexReader::checkReader(bool enforceCurrent) {
-    if (manager->getIndexMTime() > otime) {
+    if (manager->indexMTime() > otime) {
         struct timeval t;
         gettimeofday(&t, 0);
         if (enforceCurrent || t.tv_sec-otime > 60) {
@@ -170,41 +195,97 @@ CLuceneIndexReader::Private::createTerm(const wchar_t* name,
     _CLDELETE(ts);
     return t;
 }
+Term*
+CLuceneIndexReader::Private::createKeywordTerm(const wchar_t* name,
+        const string& value) {
+    wstring v = utf8toucs2(value);
+    Term* t = _CLNEW Term(name, v.c_str());
+    return t;
+}
 BooleanQuery*
-CLuceneIndexReader::Private::createBooleanQuery(const Query& query) {
+CLuceneIndexReader::Private::createBooleanQuery(const Strigi::Query& query) {
     BooleanQuery* bq = _CLNEW BooleanQuery();
-    lucene::analysis::standard::StandardAnalyzer a;
-    // add the attributes
-    const list<Query>& terms = query.getTerms();
-    list<Query>::const_iterator i;
-    set<string>::const_iterator j;
-    for (i = terms.begin(); i != terms.end(); ++i) {
-        lucene::search::Query* tq;
-        if (i->getTerms().size() > 0) {
-            tq = createBooleanQuery(*i);
-        } else {
-            wstring fieldname = mapId(i->getFieldName().c_str());
-            string expr = i->getExpression();
-            Term* t = 0;
-            if (expr.length() > 0 && expr[0] == '<') {
-                t = createTerm(fieldname.c_str(), expr.substr(1));
-                tq = _CLNEW RangeQuery(0, t, false);
-            } else if (expr.length() > 0 && expr[0] == '>') {
-                t = createTerm(fieldname.c_str(), expr.substr(1));
-                tq = _CLNEW RangeQuery(t, 0, false);
-            } else {
-                if (strpbrk(expr.c_str(), "*?")) {
-                    t = createWildCardTerm(fieldname.c_str(), expr);
-                    tq = _CLNEW WildcardQuery(t);
-                } else {
-                    t = createTerm(fieldname.c_str(), expr);
-                    tq = _CLNEW TermQuery(t);
-                }
-            }
-            if (t) _CLDECDELETE(t);
-        }
-        Query::Occurrence o = i->getOccurrence();
-        bq->add(tq, true, o == Query::MUST, o == Query::MUST_NOT);
+    bool isAnd = query.type() == Strigi::Query::And;
+    const vector<Strigi::Query>& sub = query.subQueries();
+    for (vector<Strigi::Query>::const_iterator i = sub.begin(); i != sub.end();
+            ++i) {
+        Query* q = createQuery(*i);
+        bq->add(q, true, isAnd, i->negate());
+    }
+    return bq;
+}
+Query*
+CLuceneIndexReader::Private::createQuery(const Strigi::Query& query) {
+    return query.subQueries().size()
+        ? createBooleanQuery(query)
+        : createSimpleQuery(query);
+}
+Query*
+CLuceneIndexReader::Private::createSimpleQuery(const Strigi::Query& query) {
+    switch (query.fields().size()) {
+    case 0:  return createSingleFieldQuery("content", query);//return createNoFieldQuery(query);
+    case 1:  return createSingleFieldQuery(query.fields()[0], query);
+    default: return createMultiFieldQuery(query);
+    }
+}
+Query*
+CLuceneIndexReader::Private::createSingleFieldQuery(const string& field,
+        const Strigi::Query& query) {
+    wstring fieldname = mapId(field.c_str());
+    Query* q;
+    Term* t;
+    const string& val = query.term().string();
+    switch (query.type()) {
+    case Strigi::Query::LessThan:
+          t = createTerm(fieldname.c_str(), val.c_str());
+          q = _CLNEW RangeQuery(0, t, false);
+          break;
+    case Strigi::Query::LessThanEquals:
+          t = createTerm(fieldname.c_str(), query.term().string());
+          q = _CLNEW RangeQuery(0, t, true);
+          break;
+    case Strigi::Query::GreaterThan:
+          t = createTerm(fieldname.c_str(), query.term().string());
+          q = _CLNEW RangeQuery(t, 0, false);
+          break;
+    case Strigi::Query::GreaterThanEquals:
+          t = createTerm(fieldname.c_str(), query.term().string());
+          q = _CLNEW RangeQuery(t, 0, true);
+          break;
+    case Strigi::Query::Keyword:
+          t = createKeywordTerm(fieldname.c_str(), query.term().string());
+          q = _CLNEW TermQuery(t);
+          break;	  
+    default:
+          if (strpbrk(val.c_str(), "*?")) {
+               t = createWildCardTerm(fieldname.c_str(), val);
+               q = _CLNEW WildcardQuery(t);
+          } else {
+               t = createTerm(fieldname.c_str(), val);
+               q = _CLNEW TermQuery(t);
+          }
+    }
+    _CLDECDELETE(t);
+    return q;
+}
+Query*
+CLuceneIndexReader::Private::createMultiFieldQuery(const Strigi::Query& query) {
+    BooleanQuery* bq = _CLNEW BooleanQuery();
+    for (vector<string>::const_iterator i = query.fields().begin();
+            i != query.fields().end(); ++i) {
+        Query* q = createSingleFieldQuery(*i, query);
+        bq->add(q, true, false, false);
+    }
+    return bq;
+}
+Query*
+CLuceneIndexReader::Private::createNoFieldQuery(const Strigi::Query& query) {
+    vector<string> fields = reader.fieldNames();
+    BooleanQuery* bq = _CLNEW BooleanQuery();
+    for (vector<string>::const_iterator i = fields.begin(); i != fields.end();
+            ++i) {
+        Query* q = createSingleFieldQuery(*i, query);
+        bq->add(q, true, false, false);
     }
     return bq;
 }
@@ -218,13 +299,13 @@ CLuceneIndexReader::Private::addField(lucene::document::Field* field,
 //    printf("%s\t%s\n", n.c_str(), value.c_str());
     if (wcscmp(name, L"content") == 0) {
         doc.fragment = value;
-    } else if (wcscmp(name, L"path") == 0) {
+    } else if (wcscmp(name, L"system.location") == 0) {
         doc.uri = value;
-    } else if (wcscmp(name, L"mimetype") == 0) {
+    } else if (wcscmp(name, L"content.mime_type") == 0) {
         doc.mimetype = value;
-    } else if (wcscmp(name, L"mtime") == 0) {
+    } else if (wcscmp(name, L"system.last_modified_time") == 0) {
         doc.mtime=atol(value.c_str());
-    } else if (wcscmp(name, L"size") == 0) {
+    } else if (wcscmp(name, L"system.size") == 0) {
         string size = value;
         doc.size = atoi(size.c_str());
     } else {
@@ -232,10 +313,24 @@ CLuceneIndexReader::Private::addField(lucene::document::Field* field,
             wchartoutf8(name), value));
     }
 }
+Variant
+CLuceneIndexReader::Private::getFieldValue(lucene::document::Field* field,
+        Variant::Type type) const {
+    if (field->stringValue() == 0) return Variant();
+    Variant v(wchartoutf8(field->stringValue()));
+    if (type == Variant::b_val) {
+         v = v.b();
+    } else if (type == Variant::i_val) {
+         v = v.i();
+    } else if (type == Variant::as_val) {
+         v = v.as();
+    }
+    return v;
+}
 int32_t
-CLuceneIndexReader::countHits(const Query& q) {
+CLuceneIndexReader::countHits(const Strigi::Query& q) {
     if (!checkReader()) return -1;
-    BooleanQuery* bq = Private::createBooleanQuery(q);
+    Query* bq = p->createQuery(q);
     if (reader == 0) {
         return 0;
     }
@@ -263,7 +358,7 @@ CLuceneIndexReader::countHits(const Query& q) {
         } catch (CLuceneError& err2) {
             printf("ccould not query: %s\n", err.what());
         }
-        s = counter.getCount();
+        s = counter.count();
 
         printf("counted %i hits\n", count);
         // try to do a constant score query
@@ -271,20 +366,18 @@ CLuceneIndexReader::countHits(const Query& q) {
         ConstantScoreQuery csq(filter);*/
         printf("could not query: %s\n", err.what());
     }
-    if (hits) {
-        delete hits;
-    }
+    delete hits;
     searcher.close();
     _CLDELETE(bq);
     return s;
 }
 vector<IndexedDocument>
-CLuceneIndexReader::query(const Query& q) {
+CLuceneIndexReader::query(const Strigi::Query& q, int off, int max) {
     vector<IndexedDocument> results;
     if (!checkReader()) {
         return results;
     }
-    BooleanQuery* bq = Private::createBooleanQuery(q);
+    Query* bq = p->createQuery(q);
     IndexSearcher searcher(reader);
     Hits* hits = 0;
     int s = 0;
@@ -294,11 +387,13 @@ CLuceneIndexReader::query(const Query& q) {
     } catch (CLuceneError& err) {
         printf("could not query: %s\n", err.what());
     }
-    int off = q.getOffset();
     if (off < 0) off = 0;
-    int max = q.getMax() + off;
+    max += off;
     if (max < 0) max = s;
     if (max > s) max = s;
+    if (max > off) {
+        results.reserve(max-off);
+    }
     for (int i = off; i < max; ++i) {
         Document *d = &hits->doc(i);
         IndexedDocument doc;
@@ -318,34 +413,101 @@ CLuceneIndexReader::query(const Query& q) {
     _CLDELETE(bq);
     return results;
 }
+void
+CLuceneIndexReader::getHits(const Strigi::Query& q,
+        const std::vector<std::string>& fields,
+        const std::vector<Strigi::Variant::Type>& types,
+        std::vector<std::vector<Strigi::Variant> >& result, int off, int max) {
+    result.clear();
+    if (!checkReader() || types.size() < fields.size()) {
+        return;
+    }
+
+    Query* bq = p->createQuery(q);
+    IndexSearcher searcher(reader);
+    Hits* hits = 0;
+    int s = 0;
+    try {
+        hits = searcher.search(bq);
+        s = hits->length();
+    } catch (CLuceneError& err) {
+        printf("could not query: %s\n", err.what());
+    }
+    if (off < 0) off = 0;
+    max += off;
+    if (max < 0) max = s;
+    if (max > s) max = s;
+    if (max > off) {
+        result.reserve(max-off);
+    }
+    result.resize(max-off);
+    for (int i = off; i < max; ++i) {
+        Document *d = &hits->doc(i);
+        vector<Variant>& doc = result[i-off];
+        doc.clear();
+        doc.resize(fields.size());
+
+        DocumentFieldEnumeration* e = d->fields();
+        while (e->hasMoreElements()) {
+            Field* field = e->nextElement();
+            string name(wchartoutf8(field->name()));
+            for (uint j = 0; j < fields.size(); ++j) {
+                if (fields[j] == name) {
+                    doc[j] = p->getFieldValue(field, types[j]);
+                }
+            }
+        }
+        _CLDELETE(e);
+    }
+    if (hits) {
+        _CLDELETE(hits);
+    }
+    searcher.close();
+    _CLDELETE(bq);
+}
 map<string, time_t>
-CLuceneIndexReader::getFiles(char depth) {
+CLuceneIndexReader::files(char depth) {
     map<string, time_t> files;
     if (!checkReader()) {
         return files;
     }
 
-    TCHAR tstr[CL_MAX_DIR];
+    const TCHAR* mtime = mapId(_T("system.last_modified_time"));
     char cstr[CL_MAX_DIR];
-    snprintf(cstr, CL_MAX_DIR, "%i", depth);
-    STRCPY_AtoT(tstr, cstr, CL_MAX_DIR);
+    if (depth >= 0) {
+        TCHAR tstr[CL_MAX_DIR];
+        snprintf(cstr, CL_MAX_DIR, "%i", depth);
+        STRCPY_AtoT(tstr, cstr, CL_MAX_DIR);
+        Term term(mapId(_T("system.depth")), tstr);
+        TermDocs* docs = reader->termDocs(&term);
+        while (docs->next()) {
+            Document* d = reader->document(docs->doc());
 
-    Term term(mapId(_T("depth")), tstr);
-    TermDocs* docs = reader->termDocs(&term);
-    const TCHAR* mtime = mapId(_T("mtime"));
-    while ( docs->next() ){
-        Document *d = reader->document(docs->doc());
+            const TCHAR* v = d->get(mtime);
+            STRCPY_TtoA(cstr, v, CL_MAX_DIR);
+            time_t mtime = atoi(cstr);
+            v = d->get(_T("system.location"));
+            STRCPY_TtoA(cstr, v, CL_MAX_DIR);
+            files[cstr] = mtime;
 
-        const TCHAR* v = d->get(mtime);
-        STRCPY_TtoA(cstr, v, CL_MAX_DIR);
-        time_t mtime = atoi(cstr);
-        v = d->get(_T("path"));
-        STRCPY_TtoA(cstr, v, CL_MAX_DIR);
-        files[cstr] = mtime;
-
-        _CLDELETE(d);
+            _CLDELETE(d);
+        }
+        _CLDELETE(docs);
+    } else {
+        int32_t max = reader->maxDoc();
+        for (int32_t i = 0; i < max; ++i) {
+            if (!reader->isDeleted(i)) {
+                Document* d = reader->document(i);
+                const TCHAR* v = d->get(mtime);
+                STRCPY_TtoA(cstr, v, CL_MAX_DIR);
+                time_t mtime = atoi(cstr);
+                v = d->get(_T("system.location"));
+                STRCPY_TtoA(cstr, v, CL_MAX_DIR);
+                files[cstr] = mtime;
+                _CLDELETE(d);
+            }
+        }
     }
-    _CLDELETE(docs);
     return files;
 }
 int32_t
@@ -370,17 +532,17 @@ CLuceneIndexReader::countWords() {
     return wordcount;
 }
 int64_t
-CLuceneIndexReader::getIndexSize() {
-    return manager->getIndexSize();
+CLuceneIndexReader::indexSize() {
+    return manager->indexSize();
 }
 int64_t
-CLuceneIndexReader::getDocumentId(const string& uri) {
+CLuceneIndexReader::documentId(const string& uri) {
     if (!checkReader()) return -1;
     int64_t id = -1;
 
     TCHAR tstr[CL_MAX_DIR];
     STRCPY_AtoT(tstr, uri.c_str(), CL_MAX_DIR);
-    Term term(mapId(_T("path")), tstr);
+    Term term(mapId(_T("system.location")), tstr);
     TermDocs* docs = reader->termDocs(&term);
     if (docs->next()) {
         id = docs->doc();
@@ -398,27 +560,34 @@ CLuceneIndexReader::getDocumentId(const string& uri) {
  * is not in the index, the time 0 is returned.
  **/
 time_t
-CLuceneIndexReader::getMTime(int64_t docid) {
+CLuceneIndexReader::mTime(int64_t docid) {
     if (docid < 0) return 0;
     if (!checkReader(true)) return 0;
     time_t mtime = 0;
     Document *d = reader->document(docid);
     if (d) {
         char cstr[CL_MAX_DIR];
-        const TCHAR* v = d->get(mapId(_T("mtime")));
+        const TCHAR* v = d->get(mapId(_T("system.last_modified_time")));
         STRCPY_TtoA(cstr, v, CL_MAX_DIR);
         mtime = atoi(cstr);
         delete d;
     }
     return mtime;
 }
+/**
+ * Retrieve the mtime of the document with id @docid. If this document
+ * is not in the index, the time 0 is returned.
+ **/
+time_t
+CLuceneIndexReader::mTime(const std::string& uri) {
+    return mTime(documentId(uri));
+}
 class Histogram {
 public:
     vector<pair<string,uint32_t> > h;
     vector<int> values;
 };
 #include <time.h>
-#include <sstream>
 vector<pair<string,uint32_t> >
 makeTimeHistogram(const vector<int>& v) {
     map<int32_t, int32_t> m;
@@ -426,11 +595,16 @@ makeTimeHistogram(const vector<int>& v) {
     struct tm t;
     for (i = v.begin(); i < v.end(); ++i) {
          time_t ti = *i;
+#ifdef _WIN32
+        t = *localtime( &ti );   // is thread-safe on win32
+#else
          localtime_r(&ti, &t);
+#endif
          int32_t c = 10000*t.tm_year + 100*t.tm_mon + t.tm_mday;
          m[c]++;
     }
     vector<pair<string,uint32_t> > h;
+    h.reserve(m.size());
     ostringstream str;
     map<int32_t,int32_t>::const_iterator j;
     for (j = m.begin(); j != m.end(); ++j) {
@@ -448,6 +622,7 @@ makeHistogram(const vector<int>& v, int min, int max) {
         m[*i]++;
     }
     vector<pair<string,uint32_t> > h;
+    h.reserve(m.size());
     ostringstream str;
     map<int32_t,int32_t>::const_iterator j;
     for (j = m.begin(); j != m.end(); ++j) {
@@ -458,15 +633,15 @@ makeHistogram(const vector<int>& v, int min, int max) {
     return h;
 }
 vector<pair<string,uint32_t> >
-CLuceneIndexReader::getHistogram(const string& query,
+CLuceneIndexReader::histogram(const string& query,
         const string& fieldname, const string& labeltype) {
     vector<pair<string,uint32_t> > h;
     if (!checkReader()) {
         return h;
     }
-    QueryParser parser;
-    Query q = parser.buildQuery(query, 0, 0);
-    BooleanQuery* bq = Private::createBooleanQuery(q);
+    Strigi::QueryParser parser;
+    Strigi::Query q = parser.buildQuery(query);
+    Query* bq = p->createQuery(q);
     IndexSearcher searcher(reader);
     Hits* hits = 0;
     int s = 0;
@@ -481,6 +656,7 @@ CLuceneIndexReader::getHistogram(const string& query,
     int32_t max = INT_MIN;
     int32_t min = INT_MAX;
     vector<int32_t> values;
+    values.reserve(s);
     char* end;
     for (int i = 0; i < s; ++i) {
         Document *d = &hits->doc(i);
@@ -502,14 +678,14 @@ CLuceneIndexReader::getHistogram(const string& query,
     }
     searcher.close();
     _CLDELETE(bq);
-    if (fieldname == "mtime" || labeltype == "time") {
+    if (fieldname == "system.last_modified_time" || labeltype == "time") {
         return makeTimeHistogram(values);
     } else {
         return makeHistogram(values, min, max);
     }
 }
 vector<string>
-CLuceneIndexReader::getFieldNames() {
+CLuceneIndexReader::fieldNames() {
     vector<string> s;
     if (!checkReader()) {
         return s;
@@ -521,8 +697,7 @@ CLuceneIndexReader::getFieldNames() {
         s.push_back(str);
         n++;
     }
-    // TODO: how do we clean up names?
-    // _CLDELETE(names);
+    _CLDELETE_ARRAY(names);
     return s;
 }
 int32_t
@@ -531,13 +706,13 @@ CLuceneIndexReader::countKeywords(const string& keywordprefix,
     return 2;
 }
 vector<string>
-CLuceneIndexReader::getKeywords(const string& keywordmatch,
+CLuceneIndexReader::keywords(const string& keywordmatch,
         const vector<string>& fieldnames, uint32_t max, uint32_t offset) {
     vector<string> fn;
     if (fieldnames.size()) {
         fn = fieldnames;
     } else {
-        fn = getFieldNames();
+        fn = fieldNames();
     }
     set<wstring> s;
     wstring prefix = utf8toucs2(keywordmatch);
@@ -563,7 +738,7 @@ CLuceneIndexReader::getKeywords(const string& keywordmatch,
     }
 
     vector<string> k;
-    k.resize(s.size());
+    k.reserve(s.size());
     set<wstring>::const_iterator j;
     for (j = s.begin(); j != s.end(); ++j) {
         k.push_back(wchartoutf8(*j));
