@@ -17,23 +17,26 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
-#include "jstreamsconfig.h"
+
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#include "cluceneindexwriter.h"
 #include <CLucene.h>
 #include <CLucene/store/Lock.h>
-#include "cluceneindexwriter.h"
 #include "cluceneindexreader.h"
 #include "cluceneindexmanager.h"
-#include "stringreader.h"
-#include "inputstreamreader.h"
-//comment this out if you have clucene 0.9.17 or later
-#include "PrefixFilter.h"
+#include <CLucene/util/stringreader.h>
 #include <sstream>
 #include <assert.h>
 
 #ifdef STRIGI_USE_CLUCENE_COMPRESSEDFIELDS
-#include "gzipcompressstream.h"
+#include "jsgzipcompressstream.h"
 #endif
 
+#include <iostream>
+
 using lucene::document::Document;
 using lucene::document::Field;
 using lucene::index::IndexWriter;
@@ -41,12 +44,11 @@ using lucene::index::Term;
 using lucene::index::TermDocs;
 using lucene::search::IndexSearcher;
 using lucene::search::Hits;
-using lucene::search::PrefixFilter;
 using lucene::util::BitSet;
 
 using lucene::util::Reader;
 using namespace std;
-using namespace jstreams;
+using namespace Strigi;
 
 struct CLuceneDocData {
     lucene::document::Document doc;
@@ -60,9 +62,9 @@ CLuceneIndexWriter::CLuceneIndexWriter(CLuceneIndexManager* m):
 CLuceneIndexWriter::~CLuceneIndexWriter() {
 }
 void
-CLuceneIndexWriter::addText(const Indexable* idx, const char* text,
+CLuceneIndexWriter::addText(const AnalysisResult* idx, const char* text,
         int32_t length) {
-    CLuceneDocData* doc = static_cast<CLuceneDocData*>(idx->getWriterData());
+    CLuceneDocData* doc = static_cast<CLuceneDocData*>(idx->writerData());
     doc->content.append(text, length);
 }
 
@@ -88,41 +90,68 @@ CLuceneIndexWriter::mapId(const TCHAR* id) {
     }
 }
 void
-CLuceneIndexWriter::setField(const Indexable* idx,
-        IndexerConfiguration::FieldType type, const TCHAR* name,
+CLuceneIndexWriter::addValue(const AnalysisResult* idx,
+        AnalyzerConfiguration::FieldType type, const TCHAR* name,
         const TCHAR* value) {
-    CLuceneDocData* doc = static_cast<CLuceneDocData*>(idx->getWriterData());
+    CLuceneDocData* doc = static_cast<CLuceneDocData*>(idx->writerData());
     Field* field = new Field(name, value,
-        type & IndexerConfiguration::Stored,
-        type & IndexerConfiguration::Indexed,
-        type & IndexerConfiguration::Tokenized);
+        (type & AnalyzerConfiguration::Stored) == AnalyzerConfiguration::Stored,
+        (type & AnalyzerConfiguration::Indexed) == AnalyzerConfiguration::Indexed,
+         type & AnalyzerConfiguration::Tokenized);
     doc->doc.add(*field);
 }
 void
-CLuceneIndexWriter::setField(const Indexable* idx,
-        IndexerConfiguration::FieldType type, const TCHAR* fn,
+CLuceneIndexWriter::addValue(const AnalysisResult* idx,
+        AnalyzerConfiguration::FieldType type, const TCHAR* fn,
         const std::string& value) {
 #if defined(_UCS2)
-    setField(idx, type, CLuceneIndexWriter::mapId(fn),
+    addValue(idx, type, CLuceneIndexWriter::mapId(fn),
         utf8toucs2(value).c_str());
 #else
-    setField(idx, type, CLuceneIndexWriter::mapId(fn), value.c_str());
+    addValue(idx, type, CLuceneIndexWriter::mapId(fn), value.c_str());
 #endif
 }
 void
-CLuceneIndexWriter::setField(const Indexable* idx, const string& fieldname,
-        const string& value) {
-    IndexerConfiguration::FieldType type
-        = idx->config().getIndexType(fieldname);
-    if (type == IndexerConfiguration::None) return;
+CLuceneIndexWriter::addValue(const Strigi::AnalysisResult* idx,
+        const Strigi::RegisteredField* field, const std::string& value) {
+    AnalyzerConfiguration::FieldType type
+        = idx->config().indexType(field);
+    if (type == AnalyzerConfiguration::None) return;
 #if defined(_UCS2)
-    setField(idx, type, utf8toucs2(fieldname).c_str(), value);
+    addValue(idx, type, utf8toucs2(field->key()).c_str(), value);
 #else
-    setField(idx, type, fieldname.c_str(), value);
+    addValue(idx, type, field->key(), value);
 #endif
 }
 void
-CLuceneIndexWriter::startIndexable(Indexable* idx) {
+CLuceneIndexWriter::addValue(const Strigi::AnalysisResult* idx,
+        const Strigi::RegisteredField* field, uint32_t value) {
+    ostringstream o;
+    o << value;
+    addValue(idx, field, o.str());
+}
+void
+CLuceneIndexWriter::addValue(const Strigi::AnalysisResult* idx,
+        const Strigi::RegisteredField* field, int32_t value) {
+    ostringstream o;
+    o << value;
+    addValue(idx, field, o.str());
+}
+void
+CLuceneIndexWriter::addValue(const Strigi::AnalysisResult* idx,
+        const Strigi::RegisteredField* field,
+        const unsigned char* data, uint32_t size) {
+    addValue(idx, field, string((const char*)data, (string::size_type)size));
+}
+void
+CLuceneIndexWriter::addValue(const Strigi::AnalysisResult* idx,
+        const Strigi::RegisteredField* field, double value) {
+    ostringstream o;
+    o << value;
+    addValue(idx, field, o.str());
+}
+void
+CLuceneIndexWriter::startAnalysis(const AnalysisResult* idx) {
     doccount++;
     CLuceneDocData*doc = new CLuceneDocData();
     idx->setWriterData(doc);
@@ -131,29 +160,22 @@ CLuceneIndexWriter::startIndexable(Indexable* idx) {
     Close all left open indexwriters for this path.
 */
 void
-CLuceneIndexWriter::finishIndexable(const Indexable* idx) {
-    setField(idx, "path", idx->getPath());
-    string field = idx->getEncoding();
-    if (field.length()) setField(idx, "encoding", field);
-    field = idx->getMimeType();
-    if (field.length()) setField(idx, "mimetype", field);
-    field = idx->getFileName();
-    if (field.length()) setField(idx, "filename", field);
-    field = idx->getExtension();
-    if (field.length()) setField(idx, "ext", field);
-    ostringstream o;
-    o << (int)idx->getDepth();
-    setField(idx, "depth", o.str());
-    o.str("");
-    {
-        char tmp[100];
-        snprintf(tmp,100,"%llu",(uint64_t)idx->getMTime());
-        o << tmp;
-    }
-    CLuceneDocData* doc = static_cast<CLuceneDocData*>(idx->getWriterData());
-    setField(idx, "mtime", o.str());
+CLuceneIndexWriter::finishAnalysis(const AnalysisResult* idx) {
+    const FieldRegister& fr = idx->config().fieldRegister();
+    addValue(idx, fr.pathField, idx->path());
+    string field = idx->encoding();
+    if (field.length()) addValue(idx, fr.encodingField, field);
+    field = idx->mimeType();
+    if (field.length()) addValue(idx, fr.mimetypeField, field);
+    field = idx->fileName();
+    if (field.length()) addValue(idx, fr.filenameField, field);
+    field = idx->extension();
+    if (field.length()) addValue(idx, fr.extensionField, field);
+    addValue(idx, fr.embeddepthField, idx->depth());
+    addValue(idx, fr.mtimeField, (uint32_t)idx->mTime());
+    CLuceneDocData* doc = static_cast<CLuceneDocData*>(idx->writerData());
     wstring c(utf8toucs2(doc->content));
-    StringReader<char>* sr = NULL; //we use this for compressed streams
+    jstreams::StringReader<char>* sr = NULL; //we use this for compressed streams
 
     if (doc->content.length() > 0) {
         const TCHAR* mappedFn = mapId(_T(""));
@@ -164,10 +186,10 @@ CLuceneIndexWriter::finishIndexable(const Indexable* idx) {
         // lets store the content as utf8. remember, the stream is required
         // until the document is added, so a static construction of stringreader
         // is not good enough
-        sr = new StringReader<char>(doc->content.c_str(), doc->content.length(), false);
+        sr = new jstreams::StringReader<char>(doc->content.c_str(), doc->content.length(), false);
 
     // add the stored field with the zipstream
-    doc->doc.add(*new Field(mappedFn, new GZipCompressInputStream(sr),
+    doc->doc.add(*new Field(mappedFn, new jstreams::GZipCompressInputStream(sr),
         Field::STORE_YES));
 
     // add the tokenized/indexed field
@@ -183,7 +205,7 @@ CLuceneIndexWriter::finishIndexable(const Indexable* idx) {
         try {
             writer->addDocument(&doc->doc);
         } catch (CLuceneError& err) {
-            fprintf(stderr, "%s: %s\n", idx->getPath().c_str(), err.what());
+            fprintf(stderr, "%s: %s\n", idx->path().c_str(), err.what());
         }
     }
     manager->derefWriter();
@@ -195,41 +217,47 @@ CLuceneIndexWriter::finishIndexable(const Indexable* idx) {
 void
 CLuceneIndexWriter::deleteEntries(const std::vector<std::string>& entries) {
     manager->closeWriter();
-    for (uint i=0; i<entries.size(); ++i) {
-        deleteEntry(entries[i]);
+    if (!manager->luceneReader()->checkReader()) {
+        fprintf(stderr,"cannot delete entry: lucene reader cannot be opened\n");
+        return;
     }
+    lucene::index::IndexReader* reader = manager->luceneReader()->reader;
+    
+    for (uint i=0; i<entries.size(); ++i) {
+        deleteEntry(entries[i], reader);
+    }
+    reader->commit();
     manager->setIndexMTime();
 }
 void
-CLuceneIndexWriter::deleteEntry(const string& entry) {
-    lucene::index::IndexReader* reader = manager->getReader()->reader;
+CLuceneIndexWriter::deleteEntry(const string& entry,
+        lucene::index::IndexReader* reader) {
 
     wstring tstr(utf8toucs2(entry));
-    Term term(_T("path"), tstr.c_str());
-    PrefixFilter filter(&term);
-    BitSet* bits;
-    try {
-        bits = filter.bits(reader);
-    } catch (CLuceneError& err) {
-        fprintf(stderr, "error creating filter %s: %s\n", entry.c_str(),
-            err.what());
-        bits = 0;
-    }
-    if (bits) {
-        for (int32_t i = 0; i < bits->size(); ++i) {
-            if (bits->get(i) && !reader->isDeleted(i)) {
+    int32_t prefixLen = tstr.length();
+    const TCHAR* prefixText = tstr.c_str();
+    int32_t maxdoc = reader->maxDoc();
+    for (int32_t i = 0; i < maxdoc; ++i) {
+        if (!reader->isDeleted(i)) {
+            Document* d = reader->document(i);
+            const TCHAR* t = d->get(_T("system.location"));
+            if (t && _tcsncmp(t, prefixText, prefixLen) == 0) {
                 reader->deleteDocument(i);
             }
+            _CLDELETE(d);
         }
-        _CLDELETE(bits);
     }
 }
 void
 CLuceneIndexWriter::deleteAllEntries() {
     manager->deleteIndex();
 }
+void
+CLuceneIndexWriter::commit() {
+    manager->closeWriter();
+}
 
-//this function is in 0.9.17, which we dont have yet...
+//this function is in 0.9.17, which we do not have yet...
 bool isLuceneFile(const char* filename){
     if ( !filename )
         return false;
@@ -285,19 +313,20 @@ bool isLuceneFile(const char* filename){
 
 void
 CLuceneIndexWriter::cleanUp() {
-    //remove all unused lucene file elements... unused elements are the result of unexpected shutdowns...
-    //this can add up to a lot of after a while.
+    // remove all unused lucene file elements...
+    // unused elements are the result of unexpected shutdowns...
+    // this can add up to a lot of after a while.
 
-    lucene::index::IndexReader* reader = manager->getReader()->reader;
+    lucene::index::IndexReader* reader = manager->luceneReader()->reader;
     if (!reader) {
         return;
     }
     lucene::store::Directory* directory = reader->getDirectory();
 
-    //Instantiate SegmentInfos
+    // Instantiate SegmentInfos
     lucene::store::LuceneLock* lock = directory->makeLock("commit.lock");
 #ifdef LUCENE_COMMIT_LOCK_TIMEOUT
-    //version <0.9.16
+    // version <0.9.16
     bool locked = lock->obtain(LUCENE_COMMIT_LOCK_TIMEOUT);
 #else
     bool locked = lock->obtain(lucene::index::IndexWriter::COMMIT_LOCK_TIMEOUT);
@@ -352,7 +381,22 @@ CLuceneIndexWriter::cleanUp() {
     for (i = 0; files[i] != NULL; i++) {
         _CLDELETE_CaARRAY(files[i]);
     }
-    _CLDELETE_ARRAY(files)
-
+    _CLDELETE_ARRAY(files);
 }
 
+void
+CLuceneIndexWriter::initWriterData(const FieldRegister& f) {
+    map<string, RegisteredField*>::const_iterator i;
+    map<string, RegisteredField*>::const_iterator end = f.fields().end();
+    for (i = f.fields().begin(); i != end; ++i) {
+        i->second->setWriterData(0);
+    }
+}
+void
+CLuceneIndexWriter::releaseWriterData(const FieldRegister& f) {
+    map<string, RegisteredField*>::const_iterator i;
+    map<string, RegisteredField*>::const_iterator end = f.fields().end();
+    for (i = f.fields().begin(); i != end; ++i) {
+        delete static_cast<int*>(i->second->writerData());
+    }
+}
