@@ -17,13 +17,17 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
-#include "jstreamsconfig.h"
+#include <strigi/strigiconfig.h>
 #include <CLucene.h>
-#include "indexer.h"
-#include "indexerconfiguration.h"
+#include "diranalyzer.h"
+#include "cluceneindexmanager.h"
+#include "analyzerconfiguration.h"
+#include <iostream>
 #include <sys/types.h>
 #include <stgdirent.h>
 
+using namespace std;
+
 bool
 containsHelp(int argc, char **argv) {
     for (int i=1; i<argc; ++i) {
@@ -58,9 +62,14 @@ main(int argc, char **argv) {
     vector<pair<bool,string> >filters;
     filters.push_back(make_pair<bool,string>(false,".*/"));
     filters.push_back(make_pair<bool,string>(false,".*"));
-    jstreams::IndexerConfiguration ic;
+    Strigi::AnalyzerConfiguration ic;
     ic.setFilters(filters);
-    Indexer indexer(argv[1], ic);
-    indexer.index(argv[2]);
+    try {
+        CLuceneIndexManager manager(argv[1]);
+        Strigi::DirAnalyzer analyzer(manager, ic);
+        analyzer.analyzeDir(argv[2]);
+    } catch (...) {
+        cerr << "error while creating index" << endl;
+    }
     return 0;
 }
