@@ -19,6 +19,7 @@
  */
 #include "asyncsocketclient.h"
 #include <signal.h>
+#include <stdlib.h>
 using namespace std;
 
 void
@@ -33,7 +34,7 @@ set_quit_on_signal(int signum) {
     quitaction.sa_handler = quit_daemon;
     sigaction(signum, &quitaction, 0);
 }
-
+AsyncSocket s;
 int
 main(int argc, char** argv) {
     if (argc < 2) return -1;
