@@ -26,26 +26,28 @@
 struct sqlite3;
 class SqliteIndexManager;
 
-class SqliteIndexReader : public jstreams::IndexReader {
+class SqliteIndexReader : public Strigi::IndexReader {
 friend class SqliteIndexManager;
 private:
     SqliteIndexManager* manager;
     SqliteIndexReader(SqliteIndexManager* m);
     ~SqliteIndexReader();
 public:
-    int32_t countHits(const jstreams::Query& query);
-    std::vector<jstreams::IndexedDocument> query(const jstreams::Query&);
-    std::map<std::string, time_t> getFiles(char depth);
+    int32_t countHits(const Strigi::Query& query);
+    std::vector<Strigi::IndexedDocument> query(const Strigi::Query&,
+        int off, int max);
+    std::map<std::string, time_t> files(char depth);
     int32_t countDocuments();
-    int64_t getDocumentId(const std::string& uri);
-    time_t getMTime(int64_t docid);
-    std::vector<std::pair<std::string,uint32_t> > getHistogram(
+    int64_t documentId(const std::string& uri);
+    time_t mTime(int64_t docid);
+    time_t mTime(const std::string& uri);
+    std::vector<std::pair<std::string,uint32_t> > histogram(
             const std::string& query, const std::string& fieldname,
             const std::string& labeltype);
-    std::vector<std::string> getFieldNames();
+    std::vector<std::string> fieldNames();
     int32_t countKeywords(const std::string& keywordprefix,
         const std::vector<std::string>& fieldnames);
-    std::vector<std::string> getKeywords(const std::string& keywordmatch,
+    std::vector<std::string> keywords(const std::string& keywordmatch,
         const std::vector<std::string>& fieldnames,
         uint32_t max, uint32_t offset);
 };
