@@ -1,5 +1,25 @@
-#include "jstreamsconfig.h"
+/* This file is part of Strigi Desktop Search
+ *
+ * Copyright (C) 2006 Jos van den Oever <jos@vandenoever.info>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ */
 #include "archivereader.h"
+#include <strigi/strigiconfig.h>
+#include "streambase.h"
 #include "substreamprovider.h"
 #include "tarinputstream.h"
 #include "gzipinputstream.h"
@@ -9,11 +29,14 @@
 #include "arinputstream.h"
 #include "zipinputstream.h"
 #include <vector>
-using namespace jstreams;
+#include <iostream>
+#include <cstring>
+
 using namespace std;
+using namespace Strigi;
 
 bool
-DirLister::nextEntry(jstreams::EntryInfo& e) {
+DirLister::nextEntry(EntryInfo& e) {
     if (pos < (int)entries.size()) {
         e = entries[pos++];
     } else {
@@ -21,40 +44,88 @@ DirLister::nextEntry(jstreams::EntryInfo& e) {
     }
     return pos != -1;
 }
+
+
+/**
+ * @brief A cache for entries that have already been found.
+ * @internal
+ *
+ * The cache is stored in a tree, sorted on the path in the URL
+ * (with @c / as the delimiter).
+ */
 class ArchiveEntryCache {
 public:
+    /**
+     * @brief Represents an entry in the cache.
+     */
     class SubEntry {
-    private:
-//        SubEntry(const SubEntry&);
-//        void operator=(const SubEntry&);
     public:
-        jstreams::EntryInfo entry;
+        /** The cached information about this entry. */
+        EntryInfo entry;
+        /** A list of subentries of this entry. */
         //can't define staticly constructed object while object is being defined
         std::map<std::string, SubEntry> entries;
-        int32_t getCount() const;
-        SubEntry() {
-//            entries = new std::map<std::string, SubEntry>;
-        }
-        virtual ~SubEntry() {
-//            printf("ms %i\n", entries.size());
-//            delete entries;
-        };
+        /**
+         * @brief The number of entries in the cache below and including
+         * this one.
+         *
+         * @return the size of the tree rooted at this node
+         */
+        int32_t count() const;
+        /** Constructor */
+        SubEntry() { }
+        /** Destructor */
+        virtual ~SubEntry() {}
     };
+    /**
+     * @brief Represents a top-level entry in the cache
+     */
     class RootSubEntry : public SubEntry {
-    private:
-//        RootSubEntry(const RootSubEntry&);
-//        void operator=(const RootSubEntry&);
     public:
+        /** Constructor */
         RootSubEntry() :SubEntry() {}
+        /** Whether this entry and its subentries have been indexed yet */
         bool indexed;
     };
 
+    /**
+     * @brief The cache itself.
+     *
+     * Contains all the root entries in the cache.
+     */
     std::map<std::string, RootSubEntry> cache;
-    vector<jstreams::EntryInfo> getEntries(const std::string& url);
+    // most vexing - this doesn't seem to be defined anywhere
+    // and yet... it compiles. Why? Perhaps because it isn't
+    // called from anywhere...
+    // You'd have thought you'd get a warning, though.
+    // Go figure.
+    vector<EntryInfo> entries(const std::string& url);
 
+    /**
+     * @brief Find an entry in the cache by its URL
+     *
+     * @param url the url of the entry
+     * @return a pointer to the entry, if it is in the cache,
+     * 0 if the entry is not in the cache
+     */
     const SubEntry* findEntry(const string& url) const;
+    /**
+     * @brief Finds the root subentry in the cache of the given url,
+     * if it is in the cache
+     *
+     * This finds the RootSubEntry (tree root) that the SubEntry for the
+     * given url either is or should be stored under.  If there is no
+     * suitable RootSubEntry in the cache, cache.end() is returned.
+     *
+     * @param url the url of the object to find the root entry for
+     * @return an iterator to the root entry
+     */
     map<string, RootSubEntry>::const_iterator findRootEntry(const string& url)
         const;
+    /**
+     * @brief Print a text representation of the cache to stdout.
+     * For debugging purposes.
+     */
     void print() const;
 };
 
@@ -71,29 +142,31 @@ ArchiveEntryCache::print() const {
     }
 }
 int32_t
-ArchiveEntryCache::SubEntry::getCount() const {
+ArchiveEntryCache::SubEntry::count() const {
     int32_t count = 1;
     map<string, SubEntry>::const_iterator i;
     for (i = entries.begin(); i != entries.end(); ++i) {
-        count += i->second.getCount();
+        count += i->second.count();
     }
     return count;
 }
 map<string, ArchiveEntryCache::RootSubEntry>::const_iterator
 ArchiveEntryCache::findRootEntry(const string& url) const {
     string n = url;
-    //const SubEntry* e;
     size_t p = n.size();
     do {
         map<string, RootSubEntry>::const_iterator i = cache.find(n);
         if (i != cache.end()) {
+            // it's a root entry in the cache - we're done
             return i;
         }
+        // remove the last element in the path, and look for that
         p = n.rfind('/');
         if (p != string::npos) {
             n = n.substr(0, p);
         }
     } while (p != string::npos);
+    // couldn't find it
     return cache.end();
 }
 const ArchiveEntryCache::SubEntry*
@@ -130,34 +203,47 @@ ArchiveEntryCache::findEntry(const string& url) const {
 
     return e;
 }
+
+
+/** Private data members and functions for ArchiveReader */
 class ArchiveReader::ArchiveReaderPrivate {
 public:
     class StreamPtr {
     public:
-        jstreams::StreamBase<char>* stream;
-        jstreams::SubStreamProvider* provider;
+        InputStream* stream;
+        SubStreamProvider* provider;
         StreamPtr() :stream(0), provider(0) {}
-        StreamPtr(jstreams::StreamBase<char>*s) :stream(s), provider(0) {}
-        StreamPtr(jstreams::SubStreamProvider*p) :stream(0), provider(p) {}
+        StreamPtr(InputStream* s) :stream(s), provider(0) {}
+        StreamPtr(SubStreamProvider* p) :stream(0), provider(p) {}
         void free();
     };
-    typedef std::map<jstreams::StreamBase<char>*,
+    typedef std::map<InputStream*,
         std::list<StreamPtr> > openstreamsType;
     openstreamsType openstreams;
     std::list<StreamOpener*> openers;
     ArchiveEntryCache cache;
     std::map<bool (*)(const char*, int32_t),
-        jstreams::SubStreamProvider* (*)(jstreams::StreamBase<char>*)> subs;
+        SubStreamProvider* (*)(InputStream*)> subs;
 
     std::vector<size_t> cullName(const std::string& url,
-        jstreams::StreamBase<char>*& stream) const;
-    jstreams::SubStreamProvider* getPositionedProvider(const std::string& url,
+        InputStream*& stream) const;
+    SubStreamProvider* positionedProvider(const std::string& url,
         bool onlyfiles);
-    jstreams::StreamBase<char>* open(const std::string& url) const;
-    jstreams::SubStreamProvider* getSubStreamProvider(
-        jstreams::StreamBase<char>*, std::list<StreamPtr>& streams);
+    InputStream* open(const std::string& url) const;
+    SubStreamProvider* subStreamProvider(
+        InputStream*, std::list<StreamPtr>& streams);
     static void free(std::list<StreamPtr>& l);
-    int fillEntry(ArchiveEntryCache::SubEntry&e, StreamBase<char>*s);
+    int fillEntry(ArchiveEntryCache::SubEntry&e, InputStream*s);
+    /**
+     * @brief Query known StreamOpeners about an URL.
+     *
+     * Adds the info to the cache if it was found
+     *
+     * @param url the URL to query
+     * @param e the EntryInfo structure to populate with the information
+     * @return 0 on success, -1 if the url was not found
+     */
+    int localStat(const std::string& url, EntryInfo& e);
     ArchiveReaderPrivate();
     ~ArchiveReaderPrivate();
 };
@@ -199,7 +285,7 @@ addEntry(ArchiveEntryCache::SubEntry& e, ArchiveEntryCache::SubEntry& se) {
 }
 ArchiveReader::ArchiveReaderPrivate::ArchiveReaderPrivate() {
     typedef std::pair<bool (*)(const char*, int32_t),
-        jstreams::SubStreamProvider* (*)(jstreams::StreamBase<char>*)> SubsPair;
+        SubStreamProvider* (*)(InputStream*)> SubsPair;
 
     subs.insert(SubsPair(MailInputStream::checkHeader,MailInputStream::factory) );
 
@@ -210,7 +296,7 @@ ArchiveReader::ArchiveReaderPrivate::ArchiveReaderPrivate() {
 }
 ArchiveReader::ArchiveReaderPrivate::~ArchiveReaderPrivate() {
     if (openstreams.size() > 0) {
-        fprintf(stderr, "%i streams were not closed.\n", openstreams.size());
+        cerr << openstreams.size() << " streams were not closed." << endl;
         openstreamsType::iterator i;
         for (i = openstreams.begin(); i != openstreams.end(); ++i) {
             free(i->second);
@@ -219,7 +305,7 @@ ArchiveReader::ArchiveReaderPrivate::~ArchiveReaderPrivate() {
 }
 vector<size_t>
 ArchiveReader::ArchiveReaderPrivate::cullName(const string& url,
-        StreamBase<char>*& stream) const {
+        InputStream*& stream) const {
     vector<size_t> partpos;
     size_t p = url.rfind('/');
     stream = open(url);
@@ -231,9 +317,9 @@ ArchiveReader::ArchiveReaderPrivate::cullName(const string& url,
     return partpos;
 }
 SubStreamProvider*
-ArchiveReader::ArchiveReaderPrivate::getPositionedProvider(const string& url,
+ArchiveReader::ArchiveReaderPrivate::positionedProvider(const string& url,
         bool onlyfiles) {
-    StreamBase<char>* stream = 0;
+    InputStream* stream = 0;
 
     // cull the url until a stream can be opened
     vector<size_t> partpos = cullName(url, stream);
@@ -243,21 +329,21 @@ ArchiveReader::ArchiveReaderPrivate::getPositionedProvider(const string& url,
 
     // open the substreams until we have opened the complete path
     SubStreamProvider* provider;
-    StreamBase<char>* substream = stream;
+    InputStream* substream = stream;
     vector<size_t>::reverse_iterator i;
     list<StreamPtr> streams;
     streams.push_back(stream);
     for (i = partpos.rbegin(); i != partpos.rend(); ++i) {
         const char* sn = url.c_str() + *i;
         size_t len = url.length();
-        provider = getSubStreamProvider(substream, streams);
+        provider = subStreamProvider(substream, streams);
         if (provider == 0) {
             return 0;
         }
         bool nextstream = false;
         substream = provider->currentEntry();
         do {
-            const EntryInfo& e = provider->getEntryInfo();
+            const EntryInfo& e = provider->entryInfo();
             // check that the filename matches at least one entry
             if (e.type == EntryInfo::File
                     && e.filename.length() < len
@@ -293,9 +379,9 @@ ArchiveReader::ArchiveReaderPrivate::free(list<StreamPtr>& l) {
         i->free();
     }
 }
-StreamBase<char>*
+InputStream*
 ArchiveReader::ArchiveReaderPrivate::open(const string& url) const {
-    StreamBase<char>* stream = 0;
+    InputStream* stream = 0;
     list<StreamOpener*>::const_iterator i;
     for (i = openers.begin(); i != openers.end() && stream == 0; ++i) {
         stream = (*i)->openStream(url);
@@ -303,18 +389,18 @@ ArchiveReader::ArchiveReaderPrivate::open(const string& url) const {
     return stream;
 }
 SubStreamProvider*
-ArchiveReader::ArchiveReaderPrivate::getSubStreamProvider(
-        StreamBase<char>* input, list<ArchiveReaderPrivate::StreamPtr>& streams
+ArchiveReader::ArchiveReaderPrivate::subStreamProvider(
+        InputStream* input, list<ArchiveReaderPrivate::StreamPtr>& streams
         ) {
     if (input == 0) return 0;
-    StreamBase<char>* s = new BZ2InputStream(input);
-    if (s->getStatus() == Ok) {
+    InputStream* s = new BZ2InputStream(input);
+    if (s->status() == Ok) {
         streams.push_back(s);
     } else {
         delete s;
         input->reset(0);
         s = new GZipInputStream(input);
-        if (s->getStatus() != Ok) {
+        if (s->status() != Ok) {
             delete s;
             input->reset(0);
             s = input;
@@ -327,7 +413,7 @@ ArchiveReader::ArchiveReaderPrivate::getSubStreamProvider(
     s->reset(0);
     SubStreamProvider* ss;
     map<bool (*)(const char*, int32_t),
-        SubStreamProvider* (*)(StreamBase<char>*)>::const_iterator i;
+        SubStreamProvider* (*)(InputStream*)>::const_iterator i;
     for (i = subs.begin(); i != subs.end(); ++i) {
         if (i->first(c,n)) {
             ss = i->second(s);
@@ -341,56 +427,97 @@ ArchiveReader::ArchiveReaderPrivate::getSubStreamProvider(
             s->reset(0);
         }
     }
-
+    free(streams);
     return 0;
 }
 int
 ArchiveReader::ArchiveReaderPrivate::fillEntry(ArchiveEntryCache::SubEntry& e,
-       StreamBase<char>* s) {
+       InputStream* s) {
     int nentries = 0;
     list<StreamPtr> streams;
-    SubStreamProvider* p = getSubStreamProvider(s, streams);
+    SubStreamProvider* p = subStreamProvider(s, streams);
     if (!p) return 0;
     do {
         ArchiveEntryCache::SubEntry se;
-        se.entry = p->getEntryInfo();
+        se.entry = p->entryInfo();
         int nsubentries = fillEntry(se, p->currentEntry());
         if (se.entry.size < 0) {
             // read entire stream to determine it's size
-            StreamBase<char> *es = p->currentEntry();
+            InputStream *es = p->currentEntry();
             const char* c;
             while (es->read(c, 1, 0) > 0) {}
-            se.entry.size = es->getSize();
+            se.entry.size = es->size();
             if (se.entry.size < 0) se.entry.size = 0;
         }
         addEntry(e, se);
         if (nsubentries) {
             nentries += nsubentries;
-            e.entry.type = (EntryInfo::Type)(e.entry.type|EntryInfo::Dir);
         }
+        nentries++;
     } while (p->nextEntry());
     free(streams);
+    if (nentries) {
+        e.entry.type = (EntryInfo::Type)(e.entry.type|EntryInfo::Dir);
+    }
     return nentries;
 }
+int
+ArchiveReader::ArchiveReaderPrivate::localStat(const std::string& url, EntryInfo& e) {
+    // try with the supplied streamOpeners
+    list<StreamOpener*>::const_iterator i;
+    for (i = openers.begin(); i != openers.end(); ++i) {
+        if ((*i)->stat(url, e) == 0) {
+            if (!(e.type & EntryInfo::File)) {
+                return 0;
+            }
+            // check if this file is in the cache
+            map<string, ArchiveEntryCache::RootSubEntry>::const_iterator se
+                = cache.findRootEntry(url);
+            if (se != cache.cache.end()) {
+                if (se->second.entry.mtime == e.mtime) {
+                    e.type = se->second.entry.type;
+                    return 0;
+                }
+                cache.cache.erase(se->second.entry.filename);
+            }
+
+            // we statted, now we'd like to know if this file has subentries
+            InputStream* s = (*i)->openStream(url);
+            list<ArchiveReaderPrivate::StreamPtr> streams;
+            SubStreamProvider* provider = subStreamProvider(s, streams);
+            if (provider) {
+                e.type = (EntryInfo::Type)(EntryInfo::Dir|EntryInfo::File);
+                free(streams);
+
+                ArchiveEntryCache::RootSubEntry rse;
+                rse.indexed = false;
+                rse.entry = e;
+                cache.cache[url] = rse;
+            }
+            delete s;
+            return 0;
+        }
+    }
+    return -1;
+}
 ArchiveReader::ArchiveReader() {
-    maxsize = 1024*1024;
     p = new ArchiveReaderPrivate();
 }
 ArchiveReader::~ArchiveReader() {
     delete p;
 }
 int
-ArchiveReader::stat(const std::string& url, jstreams::EntryInfo& e) {
-    if (localStat(url, e) == 0) return 0;
+ArchiveReader::stat(const std::string& url, EntryInfo& e) {
+    if (p->localStat(url, e) == 0) return 0;
 
-    // check the cache (this assumes getDirEntries was already called)
-    const ArchiveEntryCache::SubEntry *subentry = p->cache.findEntry(url);
+    // check the cache (this assumes dirEntries was already called)
+    const ArchiveEntryCache::SubEntry* subentry = p->cache.findEntry(url);
     if (subentry) {
         e = subentry->entry;
         return 0;
     }
     // try reading the entries from the collection to which this file belongs
-    getDirEntries(url);
+    dirEntries(url);
     subentry = p->cache.findEntry(url);
     if (subentry) {
         e = subentry->entry;
@@ -398,13 +525,13 @@ ArchiveReader::stat(const std::string& url, jstreams::EntryInfo& e) {
     }
     return -1;
 }
-StreamBase<char>*
+InputStream*
 ArchiveReader::openStream(const string& url) {
-    StreamBase<char>* stream = p->open(url);
+    InputStream* stream = p->open(url);
     if (stream) return stream;
 
     // open the substreams until have opened the complete path
-    SubStreamProvider* provider = p->getPositionedProvider(url, true);
+    SubStreamProvider* provider = p->positionedProvider(url, true);
     if (provider) {
         stream = provider->currentEntry();
     }
@@ -415,7 +542,7 @@ ArchiveReader::addStreamOpener(StreamOpener* opener) {
     p->openers.push_back(opener);
 }
 void
-ArchiveReader::closeStream(jstreams::StreamBase<char>* s) {
+ArchiveReader::closeStream(InputStream* s) {
     ArchiveReaderPrivate::openstreamsType::iterator i = p->openstreams.find(s);
     if (i == p->openstreams.end()) {
         delete s;
@@ -427,53 +554,14 @@ ArchiveReader::closeStream(jstreams::StreamBase<char>* s) {
 bool
 ArchiveReader::isArchive(const std::string& url) {
     EntryInfo e;
-    if (localStat(url, e) != 0) {
+    if (p->localStat(url, e) != 0) {
         return false;
     }
     return e.type & EntryInfo::File && e.type & EntryInfo::Dir;
 }
-int
-ArchiveReader::localStat(const std::string& url, jstreams::EntryInfo& e) {
-    // try with the supplied streamOpeners
-    list<StreamOpener*>::const_iterator i;
-    for (i = p->openers.begin(); i != p->openers.end(); ++i) {
-        if ((*i)->stat(url, e) == 0) {
-            if (!(e.type & EntryInfo::File)) {
-                return 0;
-            }
-            // check if this file is in the cache
-            map<string, ArchiveEntryCache::RootSubEntry>::const_iterator se
-                = p->cache.findRootEntry(url);
-            if (se != p->cache.cache.end()) {
-                if (se->second.entry.mtime == e.mtime) {
-                    e.type = se->second.entry.type;
-                    return 0;
-                }
-                p->cache.cache.erase(se->second.entry.filename);
-            }
-
-            // we statted, now we'd like to know if this file has subentries
-            StreamBase<char>* s = (*i)->openStream(url);
-            list<ArchiveReaderPrivate::StreamPtr> streams;
-            SubStreamProvider* provider = p->getSubStreamProvider(s, streams);
-            if (provider) {
-                e.type = (EntryInfo::Type)(EntryInfo::Dir|EntryInfo::File);
-                p->free(streams);
-
-                ArchiveEntryCache::RootSubEntry rse;
-                rse.indexed = false;
-                rse.entry = e;
-                p->cache.cache[url] = rse;
-            }
-            delete s;
-            return 0;
-        }
-    }
-    return -1;
-}
 DirLister
-ArchiveReader::getDirEntries(const std::string& url) {
-    std::vector<jstreams::EntryInfo> v;
+ArchiveReader::dirEntries(const std::string& url) {
+    std::vector<EntryInfo> v;
 
     // find the entry in the cache
     const ArchiveEntryCache::SubEntry *subentry = p->cache.findEntry(url);
@@ -483,6 +571,7 @@ ArchiveReader::getDirEntries(const std::string& url) {
         map<string, ArchiveEntryCache::RootSubEntry>::const_iterator se
             = p->cache.findRootEntry(url);
         if (se != p->cache.cache.end() && !se->second.indexed) {
+            // pretend like it's not in the cache
             subentry = 0;
         }
     }
@@ -491,7 +580,7 @@ ArchiveReader::getDirEntries(const std::string& url) {
         name = subentry->entry.filename;
     } else {
         // or create a new entry
-        StreamBase<char>* s = 0;
+        InputStream* s = 0;
         vector<size_t> l = p->cullName(url, s);
         if (!s) return DirLister(v);
         if (l.size()) {
@@ -500,7 +589,7 @@ ArchiveReader::getDirEntries(const std::string& url) {
             name = url;
         }
         EntryInfo e;
-        localStat(name, e);
+        p->localStat(name, e);
         ArchiveEntryCache::RootSubEntry se;
         se.indexed = true;
         se.entry = e;
@@ -522,13 +611,13 @@ ArchiveReader::getDirEntries(const std::string& url) {
 bool
 ArchiveReader::canHandle(const std::string& url) {
     vector<size_t> partpos;
-    size_t p = url.rfind('/');
+    size_t pos = url.rfind('/');
     EntryInfo e;
-    int r = localStat(url, e);
-    while (p != string::npos && p != 0 && r == -1) {
-        r = localStat(url.substr(0, p), e);
-        partpos.push_back(p+1);
-        p = url.rfind('/', p-1);
+    int r = p->localStat(url, e);
+    while (pos != string::npos && pos != 0 && r == -1) {
+        r = p->localStat(url.substr(0, pos), e);
+        partpos.push_back(pos+1);
+        pos = url.rfind('/', pos-1);
     }
     return r == 0 && e.type & EntryInfo::File && e.type & EntryInfo::Dir;
 }
