@@ -17,12 +17,14 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
-#include "jstreamsconfig.h"
 #include "arinputstream.h"
+#include <strigi/strigiconfig.h>
 #include "subinputstream.h"
 
+#include <stdlib.h>
+#include <cstring>
 
-using namespace jstreams;
+using namespace Strigi;
 
 bool
 ArInputStream::checkHeader(const char* data, int32_t datasize) {
@@ -31,28 +33,28 @@ ArInputStream::checkHeader(const char* data, int32_t datasize) {
     bool ok = memcmp(data, magic, 8) == 0;
     return ok;
 }
-ArInputStream::ArInputStream(StreamBase<char>* input)
+ArInputStream::ArInputStream(InputStream* input)
         : SubStreamProvider(input) {
     // check header
     const char*b;
     if (input->read(b, 8, 8) != 8 || !checkHeader(b, 8)) {
-        status = Error;
+        m_status = Error;
     }
 }
 ArInputStream::~ArInputStream() {
 }
-StreamBase<char>*
+InputStream*
 ArInputStream::nextEntry() {
-    if (status) return 0;
-    if (entrystream) {
-        entrystream->skip(entrystream->getSize());
-        delete entrystream;
-        entrystream = 0;
+    if (m_status) return 0;
+    if (m_entrystream) {
+        m_entrystream->skip(m_entrystream->size());
+        delete m_entrystream;
+        m_entrystream = 0;
     }
     readHeader();
-    if (status) return 0;
-    entrystream = new SubInputStream(input, entryinfo.size);
-    return entrystream;
+    if (m_status) return 0;
+    m_entrystream = new SubInputStream(m_input, m_entryinfo.size);
+    return m_entrystream;
 }
 void
 ArInputStream::readHeader() {
@@ -60,27 +62,27 @@ ArInputStream::readHeader() {
     int32_t toread;
     int32_t nread;
 
-    int64_t pos = input->getPosition();
+    int64_t pos = m_input->position();
     if (pos%2) {
-        input->skip(1);
+        m_input->skip(1);
     }
 
     // read the first 60 characters
     toread = 60;
-    nread = input->read(b, toread, toread);
-    if (input->getStatus() == Error) {
-        error = "Error reading ar header: ";
-        error += input->getError();
-        status = Error;
+    nread = m_input->read(b, toread, toread);
+    if (m_input->status() == Error) {
+        m_error = "Error reading ar header: ";
+        m_error += m_input->error();
+        m_status = Error;
         return;
     }
     if (nread <= 1) { // allow for a closing byte
-        status = Eof;
+        m_status = Eof;
         return;
     }
     if (nread != toread) {
-        error = "Error reading ar header: premature end of file.";
-        status = Error;
+        m_error = "Error reading ar header: premature end of file.";
+        m_status = Error;
         return;
     }
     int len;
@@ -91,27 +93,47 @@ ArInputStream::readHeader() {
         }
     }
 
-    entryinfo.size = atoi(b+48);
-    entryinfo.mtime = atoi(b+16);
+    // we must copy this string to safely call atoi
+    char bc[61];
+    memcpy(bc, b, 60);
+    bc[60] = '\0';
+    m_entryinfo.size = atoi(bc+48);
+    if (m_entryinfo.size < 0) {
+        m_error = "Error: negative file size.";
+        m_status = Error;
+        return;
+    }
+    m_entryinfo.mtime = atoi(bc+16);
     if (len == 0) {
         if (b[1] == '/') {
-            nread = input->read(b, entryinfo.size, entryinfo.size);
-            gnufilenames = std::string(b, entryinfo.size);
+            // fprintf(stderr, "SIZE: %lli\n", m_entryinfo.size);
+            nread = m_input->read(b, m_entryinfo.size, m_entryinfo.size);
+            if (nread != m_entryinfo.size) {
+                m_error = "premature end of stream";
+                m_status = Error;
+                return;
+            }
+            gnufilenames.assign(b, m_entryinfo.size);
             readHeader();
         } else if (b[1] == ' ') {
-            input->skip(entryinfo.size);
+            m_input->skip(m_entryinfo.size);
             readHeader();
         } else {
-            int p = atoi(b+1);
+            uint p = atoi(bc+1);
+            if (gnufilenames.length() <= p) {
+                m_error = "Invalid name field.";
+                m_status = Error;
+                return;
+            }
             const char* c = gnufilenames.c_str() + p;
             const char* e = strchr(c, '/');
             if (e) {
-                entryinfo.filename = std::string(c, e-c);
+                m_entryinfo.filename = std::string(c, e-c);
             } else {
-                entryinfo.filename = c;
+                m_entryinfo.filename = c;
             }
         }
     } else {
-        entryinfo.filename = std::string(b, len);
+        m_entryinfo.filename = std::string(b, len);
     }
 }
