@@ -22,24 +22,24 @@
 
 #include "substreamprovider.h"
 
-namespace jstreams {
+namespace Strigi {
 
-class ArInputStream : public SubStreamProvider {
+class STREAMS_EXPORT ArInputStream : public SubStreamProvider {
 private:
     // information relating to the current entry
     std::string gnufilenames;
 
     void readHeader();
 public:
-    explicit ArInputStream(StreamBase<char>* input);
+    explicit ArInputStream(InputStream* input);
     ~ArInputStream();
-    StreamBase<char>* nextEntry();
+    InputStream* nextEntry();
     static bool checkHeader(const char* data, int32_t datasize);
-    static SubStreamProvider* factory(StreamBase<char>* input) {
+    static SubStreamProvider* factory(InputStream* input) {
         return new ArInputStream(input);
     }
 };
 
-} // end namespace jstreams
+} // end namespace Strigi
 
 #endif
