@@ -22,29 +22,19 @@
 
 #include "bufferedstream.h"
 
-namespace jstreams {
-class Base64InputStream : public BufferedInputStream<char> {
+namespace Strigi {
+
+class STREAMS_EXPORT Base64InputStream : public BufferedInputStream {
 private:
-    int32_t bits;
-    char bytestodo;
-    char char_count;
-    const char* pos, * pend;
-    int32_t nleft;
-
-    StreamBase<char>* input;
-
-    static const unsigned char alphabet[];
-    static bool inalphabet[256];
-    static unsigned char decoder[133];
-    static bool initializedAlphabet;
-    static void initialize();
-
-    bool moreData();
-public:
-    explicit Base64InputStream(StreamBase<char>* i);
+    class Private;
+    Private* const p;
     int32_t fillBuffer(char* start, int32_t space);
-    static std::string decode(const char*c, std::string::size_type length);
+public:
+    explicit Base64InputStream(InputStream* i);
+    ~Base64InputStream();
+    static std::string decode(const char* in, std::string::size_type length);
 };
-}
+
+} // end namespace Strigi
 
 #endif
