@@ -22,27 +22,19 @@
 
 #include "bufferedstream.h"
 
-#include <bzlib.h>
+namespace Strigi {
 
-namespace jstreams {
-
-class BZ2InputStream : public BufferedInputStream<char> {
+class STREAMS_EXPORT BZ2InputStream : public BufferedInputStream {
 private:
-    bool allocatedBz;
-    bz_stream* bzstream;
-    StreamBase<char> *input;
-
-    void dealloc();
-    void readFromStream();
-    bool checkMagic();
-protected:
+    class Private;
+    Private* const p;
     int32_t fillBuffer(char* start, int32_t space);
 public:
-    BZ2InputStream(StreamBase<char>* input);
+    BZ2InputStream(InputStream* input);
     ~BZ2InputStream();
     static bool checkHeader(const char* data, int32_t datasize);
 };
 
-} // end namespace jstreams
+} // end namespace Strigi
 
 #endif
