@@ -22,22 +22,38 @@
 
 #include "bufferedstream.h"
 
-namespace jstreams {
+namespace Strigi {
 
-class FileInputStream : public BufferedInputStream<char> {
+/**
+ * @brief Provides buffered access to a file
+ */
+class STREAMS_EXPORT FileInputStream : public BufferedInputStream {
 private:
     FILE *file;
     std::string filepath;
 
-public:
+    int32_t fillBuffer(char* start, int32_t space);
+    /** The default buffer size, only used as a default argument to the constructor */
     static const int32_t defaultBufferSize;
-    explicit FileInputStream(const char *filepath,
+protected:
+    void open(FILE* file, const char* filepath,
+        int32_t buffersize=defaultBufferSize);
+
+public:
+    /**
+     * @brief Create an InputStream to access a file
+     *
+     * @param filepath the name of the file to open
+     * @param buffersize the size of the buffer to use
+     */
+    explicit FileInputStream(const char* filepath,
+        int32_t buffersize=defaultBufferSize);
+    FileInputStream(FILE* file, const char* filepath,
         int32_t buffersize=defaultBufferSize);
     ~FileInputStream();
-    int32_t fillBuffer(char* start, int32_t space);
 };
 
-} // end namespace jstreams
+} // end namespace Strigi
 
 #endif
 
