@@ -22,27 +22,20 @@
 
 #include "bufferedstream.h"
 
-struct z_stream_s;
+namespace Strigi {
 
-namespace jstreams {
-
-class GZipInputStream : public BufferedInputStream<char> {
+class STREAMS_EXPORT GZipInputStream : public BufferedInputStream {
 private:
-    z_stream_s* zstream;
-    StreamBase<char>* input;
-
-    void dealloc();
-    void readFromStream();
-    void decompressFromStream();
-    bool checkMagic();
+    class Private;
+    Private* const p;
+    int32_t fillBuffer(char* start, int32_t space);
 public:
     enum ZipFormat { ZLIBFORMAT, GZIPFORMAT, ZIPFORMAT};
-    explicit GZipInputStream(StreamBase<char>* input,
+    explicit GZipInputStream(InputStream* input,
         ZipFormat format=GZIPFORMAT);
     ~GZipInputStream();
-    int32_t fillBuffer(char* start, int32_t space);
 };
 
-} // end namespace jstreams
+} // end namespace Strigi
 
 #endif
