@@ -20,64 +20,73 @@
 #ifndef MAILINPUTSTREAM_H
 #define MAILINPUTSTREAM_H
 
+#include <strigi/strigiconfig.h>
+#include "streambase.h"
 #include "substreamprovider.h"
+#include <stack>
+
+namespace Strigi {
+
+class StringTerminatedSubStream;
 
 /**
- * This is a proof of concept implementation for handling email streams as
+ * This is an implementation for handling email streams as
  * archives. It allows one to read the email body and email attachements as
  * streams.
  **/
-
-namespace jstreams {
-class SubInputStream;
-class StringTerminatedSubStream;
-class MailInputStream : public SubStreamProvider {
+class STREAMS_EXPORT MailInputStream : public SubStreamProvider {
 private:
+    int64_t nextLineStartPosition;
     // variables that record the current read state
-    bool eol; // true if the buffer contains a line end
     int32_t entrynumber;
-    int64_t bufstartpos;
-    int64_t bufendpos;
-    int linenum;
     int maxlinesize;
     const char* linestart;
     const char* lineend;
-    const char* bufstart;
-    const char* bufend;
 
     StringTerminatedSubStream* substream;
-    std::string subject;
-    std::string contenttype;
+    std::string m_subject;
+    std::string m_from;
+    std::string m_to;
+    std::string m_cc;
+    std::string m_bcc;
+    std::string m_messageid;
+    std::string m_inreplyto;
+    std::string m_references;
+    std::string m_contenttype;
     std::string contenttransferencoding;
     std::string contentdisposition;
-    std::string* lastHeader;
 
-    std::string boundary;
+    std::stack<std::string> boundary;
 
-    void readLine();
-    void fillBuffer();
-    void skipHeader();
+    void readHeaderLine();
+    void readHeader();
     void scanBody();
     void handleHeaderLine();
-    void handleBodyLine();
+    bool handleBodyLine();
     bool lineIsEndOfBlock();
-    void rewindToLineStart();
     bool checkHeaderLine() const;
     void clearHeaders();
     void ensureFileName();
-    std::string getValue(const char* n, const std::string& headerline) const;
+    std::string value(const char* n, const std::string& headerline) const;
 public:
-    explicit MailInputStream(StreamBase<char>* input);
+    explicit MailInputStream(InputStream* input);
     ~MailInputStream();
-    StreamBase<char>* nextEntry();
+    InputStream* nextEntry();
     static bool checkHeader(const char* data, int32_t datasize);
-    static SubStreamProvider* factory(StreamBase<char>* input) {
+    static SubStreamProvider* factory(InputStream* input) {
         return new MailInputStream(input);
     }
-    const std::string& getSubject() { return subject; }
-    const std::string& getContentType() { return contenttype; }
+    const std::string& subject() { return m_subject; }
+    const std::string& from() { return m_from; }
+    const std::string& to() { return m_to; }
+    const std::string& cc() { return m_cc; }
+    const std::string& bcc() { return m_bcc; }
+    const std::string& messageid() { return m_messageid; }
+    const std::string& inreplyto() { return m_inreplyto; }
+    const std::string& references() { return m_references; }
+    const std::string& contentType() { return m_contenttype; }
 };
 
-} // end namespace jstreams
+} // end namespace Strigi
 
 #endif
