@@ -1,11 +1,30 @@
-#include "jstreamsconfig.h"
+/* This file is part of Strigi Desktop Search
+ *
+ * Copyright (C) 2006 Jos van den Oever <jos@vandenoever.info>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; see the file COPYING.LIB.  If not, write to
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ */
 #include "pdfparser.h"
 #include "stringterminatedsubstream.h"
 #include "gzipinputstream.h"
 #include "subinputstream.h"
 #include <ctype.h>
-using namespace jstreams;
+
 using namespace std;
+using namespace Strigi;
 
 int32_t streamcount = 0;
 
@@ -15,22 +34,22 @@ PdfParser::PdfParser() :streamhandler(0), texthandler(0) {
 StreamStatus
 PdfParser::read(int32_t min, int32_t max) {
     int32_t off = pos-start;
-    int32_t d = stream->getPosition() - objdefstart;
+    int32_t d = stream->position() - objdefstart;
     min += d;
     if (max > 0) max += d;
     stream->reset(objdefstart);
     int32_t n = stream->read(start, min, max);
 //    printf("objstart %i %i\n", d, n);
-    if (n < min) return stream->getStatus();
+    if (n < min) return stream->status();
     pos = start + off;
     end = start + n;
     return Ok;
 }
 StreamStatus
 PdfParser::read2(int32_t min, int32_t max) {
-    printf("pos %lli\n", stream->getPosition());
+//    printf("pos %lli\n", stream->position());
     int32_t n = stream->read(start, min, max);
-    if (n < min) return stream->getStatus();
+    if (n < min) return stream->status();
     pos = start;
     end = pos + n;
     return Ok;
@@ -41,6 +60,7 @@ PdfParser::checkForData(int32_t m) {
     if (end - pos < m) {
         n = read(m, 0);
     }
+//    fprintf(stderr, "checkForData %i\n", n);
     return n;
 }
 
@@ -52,6 +72,24 @@ PdfParser::isInString(char c, const char* s, int32_t n) {
     return false;
 }
 StreamStatus
+PdfParser::skipDigits() {
+    StreamStatus s;
+    do {
+        if ((s = checkForData(1)) != Ok) return s;
+        while (pos < end && isdigit(*pos)) pos++;
+    } while (pos == end);
+    return Ok;
+}
+StreamStatus
+PdfParser::skipXChars() {
+    StreamStatus s;
+    do {
+        if ((s = checkForData(1)) != Ok) return s;
+        while (pos < end && isxdigit(*pos)) pos++;
+    } while (pos == end);
+    return Ok;
+}
+StreamStatus
 PdfParser::skipFromString(const char*str, int32_t n) {
     StreamStatus s;
     do {
@@ -74,15 +112,15 @@ PdfParser::skipKeyword(const char* str, int32_t len) {
     if (end - pos < len) {
         StreamStatus s;
         if ((s = read(len, 0)) != Ok) {
-            error.assign("Premature end of stream.");
+            m_error.assign("Premature end of stream.");
             return Error;
         }
     }
 //    printf("skipKeyword %s '%.*s'\n", str, (len>end-pos)?end-pos:len, pos);
     if (strncmp(pos, str, len) != 0) {
-            error.assign("Keyword ");
-            error.append(str, len);
-            error.append(" not found.");
+            m_error.assign("Keyword ");
+            m_error.append(str, len);
+            m_error.append(" not found.");
             return Error;
     }
     pos += len;
@@ -91,11 +129,16 @@ PdfParser::skipKeyword(const char* str, int32_t len) {
 /**
  * Skip whitespace in the stream. Return amount of whitespace skipped.
  * After calling this function the position in the stream is after the
- * whitespace.
+ * whitespace. Skip characters from \t\n\f\r\t and space.
  **/
 StreamStatus
 PdfParser::skipWhitespace() {
-    return skipFromString("\t\n\f\r ", 6);
+    StreamStatus s;
+    do {
+        if ((s = checkForData(1)) != Ok) return s;
+        while (pos < end && isspace(*pos)) pos++;
+    } while (pos == end);
+    return Ok;
 }
 StreamStatus
 PdfParser::parseComment() {
@@ -121,26 +164,38 @@ StreamStatus
 PdfParser::parseBoolean() {
     return (*pos == 't') ?skipKeyword("true", 4) :skipKeyword("false", 5);
 }
-// - number : [+-]?\d+(.\d+)?
 StreamStatus
-PdfParser::parseNumber() {
-//    printf("parseNumber\n");
-    static const char* nc = "0123456789";
-    int64_t p = pos - start;
+PdfParser::skipNumber() {
     char ch = *pos;
     if (ch == '+' || ch == '-') pos++;
-    StreamStatus n = skipFromString(nc, 10);
+    StreamStatus n = skipDigits();
     if (n != Ok) return n;
     if (pos < end && *pos == '.') {
         pos++;
-        n = skipFromString(nc, 10);
+        n = skipDigits();
     }
-    const char *s = start + p;
-    // parse an integer
-    lastNumber = strtod(s, 0);
+    return n;
+}
+// - number : [+-]?\d+(.\d+)?
+StreamStatus
+PdfParser::parseNumber() {
+    int64_t p = pos - start;
+    char ch = *pos;
+    if (ch == '+' || ch == '-') pos++;
+    StreamStatus n = skipDigits();
+    if (n != Ok) return n;
+    if (pos < end && *pos == '.') {
+        pos++;
+        n = skipDigits();
+        const char *s = start + p;
+        lastNumber = strtod(s, 0);
+    } else {
+        const char *s = start + p;
+        lastNumber = strtol(s, 0, 10);
+    }
+    // parse a number
     lastObject = &lastNumber;
     if (lastNumber > 300 || lastNumber < -300) lastString.append(" ", 1);
-//    printf("%f\n", lastNumber);
     return n;
 }
 StreamStatus
@@ -205,11 +260,14 @@ PdfParser::parseLiteralString() {
 }
 StreamStatus
 PdfParser::parseHexString() {
-//    printf("parseHexString\n");
-    if (skipNotFromString(">", 1) != Ok) {
-         error.assign("Premature end of stream while parsing a hexstring.");
+    skipKeyword("<", 1);
+//    fprintf(stderr, "parseHexString\n");
+    if (skipXChars() != Ok) {
+//         fprintf(stderr, "not a hex string\n");
+         m_error.assign("invalid hexstring.");
          return Error;
     }
+//    printf("parseHexString ok\n");
     return skipKeyword(">", 1);
 }
 StreamStatus
@@ -219,7 +277,7 @@ PdfParser::parseName() {
     int64_t p = pos - start;
     StreamStatus r = skipNotFromString("()<>[]{}/%\t\n\f\r ", 16);
     if (r == Error) {
-        error.assign(stream->getError());
+        m_error.assign(stream->error());
         return r;
     }
     const char *s = start + p;
@@ -232,13 +290,15 @@ PdfParser::parseOperator() {
     int64_t p = pos - start;
     StreamStatus r = skipNotFromString("()<>[]{}/%\t\n\f\r ", 16);
     if (r == Error) {
-        error.assign(stream->getError());
+        m_error.assign(stream->error());
         return r;
     }
     const char *s = start + p;
     lastOperator.assign(s, pos-s);
     if (lastOperator == "TJ" || lastOperator == "Tj") {
-        printf("%s\n", lastString.c_str());
+        if (texthandler) {
+            texthandler->handle(lastString);
+        }
         lastString.resize(0);
     }
     lastObject = &lastOperator;
@@ -248,7 +308,7 @@ StreamStatus
 PdfParser::parseDictionaryOrStream() {
     enum Mode {None, Length, Filter, Type, First, N};
     Mode mode = None;
-//    printf("parseDictionary %p\n", this);
+//    fprintf(stderr, "parseDictionary %p\n", this);
     StreamStatus r;
     pos += 2;
     skipWhitespaceOrComment();
@@ -260,7 +320,7 @@ PdfParser::parseDictionaryOrStream() {
     int numberofobjects = 0;
     while (*pos != '>') {
         if (parseName() != Ok) {
-            error.assign("Expected a name.");
+            m_error.assign("Expected a name.");
             return Error;
         }
         if (lastName == "Length") mode = Length;
@@ -270,12 +330,12 @@ PdfParser::parseDictionaryOrStream() {
         else if (lastName == "N") mode = N;
         else mode = None;
         if (skipWhitespaceOrComment() != Ok) {
-            error.assign("Error parsing whitespace in dictionary.");
+            m_error.assign("Error parsing whitespace in dictionary.");
             return Error;
         }
         lastObject = 0;
         if (parseObjectStreamObject() != Ok) {
-            error.assign("Error parsing dictionary value.");
+            m_error.assign("Error parsing dictionary value.");
             return Error;
         }
         if (mode == Length && lastObject == &lastNumber) {
@@ -293,15 +353,15 @@ PdfParser::parseDictionaryOrStream() {
             numberofobjects = (int32_t)lastNumber;
         }
         if (skipWhitespaceOrComment() != Ok) {
-            error.assign("Error reading whitespace after dictionary value.");
+            m_error.assign("Error reading whitespace after dictionary value.");
             return Error;
         }
     }
     if (skipKeyword(">>", 2) != Ok) return Error;
     r = skipWhitespaceOrComment();
     if (r != Ok) return r;
-    if (*pos == 's') {
-//        printf("stream %i\n", end-pos);
+    if (checkForData(6) == Ok && *pos == 's' && pos[2] == 'r') {
+//        fprintf(stderr, "stream %i\n", end-pos);
         skipKeyword("stream", 6);
         if (checkForData(11) != Ok) return Error;
         if (*pos == '\r') pos++;
@@ -311,27 +371,28 @@ PdfParser::parseDictionaryOrStream() {
         // read stream until 'endstream'
         int64_t p = pos-start;
         if (p != stream->reset(p)) return Error;
-//        printf("filter: %s\n", filter.c_str());
-//        printf("position: %i\n", p);
+//        fprintf(stderr, "filter: %s\n", filter.c_str());
+//        fprintf(stderr, "type: %s %i\n", type.c_str(), streamcount);
+//        printf("position: %lli length %i\n", p, length);
         if (length == -1) {
             StringTerminatedSubStream sub(stream, "endstream");
             if (handleSubStream(&sub, type, offset, numberofobjects, hasfilter,
                     filter) != Eof) {
                 return Error;
             }
-            p += sub.getSize();
+            p += sub.size();
         } else {
             SubInputStream sub(stream, length);
             if (handleSubStream(&sub, type, offset, numberofobjects, hasfilter,
                     filter) != Eof) {
                 return Error;
             }
-            p += sub.getSize();
+            p += sub.size();
         }
         if (p != stream->reset(p)) return Error;
         if (read(1, 0) != Ok) return Error;
         pos = start + p;
-        //pos = start + (stream->getPosition()-objdefstart);
+        //pos = start + (stream->position()-objdefstart);
 //        printf("hi %i\n", off+(pos-start));
         //printf("hi %i\n", *pos);
         if (skipWhitespaceOrComment() != Ok) return Error;
@@ -345,7 +406,6 @@ PdfParser::parseDictionaryOrStream() {
 }
 StreamStatus
 PdfParser::parseArray() {
-//    printf("parseArray\n");
     lastString.resize(0);
     pos++;
     if (skipWhitespaceOrComment() != Ok) return Error;
@@ -395,10 +455,11 @@ PdfParser::parseObjectStreamObject() {
 }
 StreamStatus
 PdfParser::parseContentStreamObject() {
-//    printf("parseContentStreamObject %.*s\n", (5>end-pos)?end-pos:5, pos);
-//    printf("pos: %i\n", stream->getPosition());
     StreamStatus r = read(2, 0);
     if (r == Error) return r;
+//    fprintf(stderr, "parseContentStreamObject %.*s\n",
+//        (5>end-pos)?end-pos:5, pos);
+//    fprintf(stderr, "pos: %lli\n", stream->position());
 
     char ch = *pos;
     if (ch == '+' || ch == '-' || ch == '.' || isdigit(ch)) {
@@ -416,7 +477,6 @@ PdfParser::parseContentStreamObject() {
     } else if (ch == '[') {
         r = parseArray();
     } else if (isalpha(ch)) {
-//        printf("jaaj%i\n", ch);
         r = parseOperator();
     } else {
         return Error;
@@ -438,10 +498,10 @@ PdfParser::parseObjectStream(StreamBase<char>* s, int32_t offset, int32_t n) {
     }
     while (r == Ok) {
         s->skip(1000);
-        r = s->getStatus();
+        r = s->status();
     }
     if (r == Eof) {
-//        printf("size: %i\n", s->getSize());
+//        printf("size: %i\n", s->size());
     }
     return r;
 }
@@ -451,19 +511,57 @@ PdfParser::parseContentStream(StreamBase<char>* s) {
     end = pos = start = 0;
     objdefstart = 0;
     StreamStatus r = skipWhitespaceOrComment();
+//    fprintf(stderr, "eh %i %i\n", r, Eof);
     if (r != Ok) return r;
     while ((r = parseContentStreamObject()) == Ok) {};
-//    printf("%i %i\n", r, streamcount);
-    printf("parseContentStream done\n");
+//    fprintf(stderr, "PdfParser::parseContentStream %i %i\n", r, Eof);
     return r;
 }
 StreamStatus
+PdfParser::skipXRef() {
+    // skip header
+    if (skipKeyword("xref", 4) != Ok || skipWhitespaceOrComment() != Ok
+            || skipNumber() != Ok || skipWhitespaceOrComment() != Ok
+            || parseNumber() != Ok || skipWhitespaceOrComment() != Ok) {
+        return Error;
+    }
+    // parse number of entreis
+    int entries = (int)lastNumber;
+    for (int i = 0; i != entries; ++i) {
+        if (skipNumber() != Ok || skipWhitespaceOrComment() != Ok
+            || skipNumber() != Ok || skipWhitespaceOrComment() != Ok
+            || skipFromString("fn", 2) != Ok
+            || skipWhitespaceOrComment() != Ok) {
+                return Error;
+        }
+    }
+    return Ok;
+}
+StreamStatus
+PdfParser::skipTrailer() {
+    if (skipKeyword("trailer", 7) != Ok || skipWhitespaceOrComment() != Ok
+            || parseDictionaryOrStream() != Ok) {
+        return Error;
+    }
+    return Ok;
+}
+StreamStatus
+PdfParser::skipStartXRef() {
+    if (skipKeyword("startxref", 9) != Ok || skipWhitespaceOrComment() != Ok
+            || skipNumber() != Ok) {
+        fprintf(stderr, "error in startxref 1\n");
+        return Error;
+    }
+    return skipWhitespaceOrComment();
+}
+StreamStatus
 PdfParser::parseObjectStreamObjectDef() {
 //    objdefstart = pos-start;
-    // if we are at 'xref' or 'startxref' we're ready
-    if (*pos == 'x' || *pos == 's') return Eof;
+    if (*pos == 'x') return skipXRef();
+    if (*pos == 't') return skipTrailer();
+    if (*pos == 's') return skipStartXRef();
     if (checkForData(13) != Ok) return Error;
-//    printf("parseObjectStreamObjectDef %.*s\n", ((10>end-pos)?end-pos:10), pos);
+//    fprintf(stderr, "parseObjectStreamObjectDef %.*s\n", ((10>end-pos)?end-pos:10), pos);
     if (parseNumber() != Ok || skipWhitespaceOrComment() != Ok
         || parseNumber() != Ok || skipWhitespaceOrComment() != Ok
         || skipKeyword("obj", 3) != Ok || skipWhitespaceOrComment() != Ok
@@ -471,19 +569,17 @@ PdfParser::parseObjectStreamObjectDef() {
         || skipKeyword("endobj", 6) != Ok) {
         return Error;
     }
-
-    return skipWhitespaceOrComment();
+    StreamStatus r = skipWhitespaceOrComment();
+//    fprintf(stderr, "parsed obj ok %i\n", r);
+    return r;
 }
 StreamStatus
 PdfParser::parse(StreamBase<char>* stream) {
     // for now we need to load the entire stream in memory :(
     // this is due to a sneaky bug somewhere, not a design issue
-    const char* c;
-    //warning:use limited to sized streams
-    stream->read(c, 1+stream->getSize(), 0);
+    forwardStream(stream);
     stream->reset(0);
-    StreamStatus r = stream->getStatus();
-    if (r != Ok) return r;
+    StreamStatus r;
 
     // initialize the stream status
     this->stream = stream;
@@ -496,32 +592,50 @@ PdfParser::parse(StreamBase<char>* stream) {
     lastObject = 0;
 
     r = skipWhitespaceOrComment();
-    if (r != Ok) return r;
+    if (r != Ok) {
+        fprintf(stderr, "Error: %s\n", stream->error());
+        return r;
+    }
     while ((r = parseObjectStreamObjectDef()) == Ok) {};
-//    printf("%i %i\n", r, streamcount);
+//    fprintf(stderr, "%i %i %i\n", r, streamcount, Eof);
+    if (r == Error) {
+        fprintf(stderr, "Error in parsing: %s\n", m_error.c_str());
+    }
     return r;
 }
-#include <cstdio>
-void
-saveFile(const char* start, const char* end, const char* suf=0) {
+Strigi::StreamStatus
+PdfParser::DefaultStreamHandler::handle(Strigi::StreamBase<char>* s) {
     static int count = 0;
     char name[32];
-    if (suf) {
-        sprintf(name, "out/%i.%s", ++count, suf);
-    } else {
-        sprintf(name, "out/%i", ++count);
+    const char *c;
+    int32_t n = s->read(c, 1, 0);
+    if (n <= 0) {
+        return s->status();
     }
+    sprintf(name, "out/%i", ++count);
     FILE* file = fopen(name, "wb");
-    if (file == 0) return;
-    fwrite(start, 1, end-start, file);
+    if (file == 0) {
+        return Error;
+    }
+    do {
+        fwrite(c, 1, n, file);
+        n = s->read(c, 1, 0);
+    } while (n > 0);
     fclose(file);
+    return s->status();
+}
+Strigi::StreamStatus
+PdfParser::DefaultTextHandler::handle(const string& s) {
+    printf("%s\n", s.c_str());
+    return Ok;
 }
 void
 PdfParser::forwardStream(StreamBase<char>* s) {
     const char* c;
     int32_t n = s->read(c, 1024, 0);
-    while (n >= 0 && s->getStatus() == Ok) {
-        n = s->read(c, n, 0);
+    while (n >= 0 && s->status() == Ok) {
+        s->reset(0);
+        n = s->read(c, 2*n, 0);
     }
 }
 StreamStatus
@@ -533,9 +647,8 @@ PdfParser::handleSubStream(StreamBase<char>* s, const std::string& type,
             GZipInputStream gzip(s, GZipInputStream::ZLIBFORMAT);
             return handleSubStream(&gzip, type, offset, numberofobjects);
         } else {
-            // we cannot handle these filters, so we skip this data
-            forwardStream(s);
-            return s->getStatus();
+            // we cannot handle these filters, so we send the raw data
+            return handleSubStream(s, type, 0, 0);
         }
     } else {
         return handleSubStream(s, type, offset, numberofobjects);
@@ -544,46 +657,28 @@ PdfParser::handleSubStream(StreamBase<char>* s, const std::string& type,
 StreamStatus
 PdfParser::handleSubStream(StreamBase<char>* s, const std::string& type,
         int32_t offset, int32_t numberofobjects) {
-    const char* c;
-    int32_t n = s->read(c, 1024, 0);
-    while (n >= 0 && s->getStatus() == Ok) {
-        n = s->read(c, n, 0);
-    }
-    s->reset(0);
-
     // try to parse as an object stream
     PdfParser parser;
+    parser.texthandler = texthandler;
+    parser.streamhandler = streamhandler;
     if (type == "ObjStm") {
-//        printf("object stream\n");
         if (parser.parseObjectStream(s, offset, numberofobjects) == Eof) {
-//            printf("Found object stream %i of length %i.\n",
-//                count++, s->getSize());
             return Eof;
         } else {
-//            printf("%s\n", parser.getError().c_str());
-//            printf("%.*s\n", 100, c+offset);
             return Error;
         }
     }
 
     // try to parse as a content stream
+    s->reset(0);
     if (parser.parseContentStream(s) == Eof) {
-//        printf("Found content stream %i of length %i.\n",
-//            count++, s->getSize());
         return Eof;
     }
-
-/*    if (n > 10) {
-        bool allascii = true;
-        for (int i=0; i<10; ++i) {
-            allascii &= isascii(c[i]);
-        }
-//        if (!allascii) {
-//        }
-    }*/
-            saveFile(c, c+n);
-//    s->read(c, s->getSize()+1, 0);
-//    printf("status: %i\n", s->getStatus());
+    // handle the stream by an external handler
+    s->reset(0);
+    if (streamhandler) {
+        streamhandler->handle(s);
+    }
     forwardStream(s);
-    return s->getStatus();
+    return s->status();
 }
