@@ -17,20 +17,21 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
-#include "jstreamsconfig.h"
 #include "substreamproviderprovider.h"
+#include <strigi/strigiconfig.h>
 #include "mailinputstream.h"
 #include "tarinputstream.h"
 #include "zipinputstream.h"
-using namespace jstreams;
+
+using namespace Strigi;
 
 SubStreamProviderProvider::SubStreamProviderProvider() {
 }
 SubStreamProvider*
-SubStreamProviderProvider::getSubStreamProvider(StreamBase<char>* input) {
+SubStreamProviderProvider::subStreamProvider(InputStream* input) {
     // read the header
     const char* header;
-    int64_t pos = input->getPosition();
+    int64_t pos = input->position();
     int32_t headersize = input->read(header, 1024, 0);
     input->reset(pos);
     if (headersize <= 0) {
